from gnssrefl.rinex2snr import *
from gnssrefl.gps import *


REFL_CODE = os.environ["REFL_CODE"]

def test_quickname():
    assert (
        quickname("p103", 2020, "20", "105", "99")
        == f"{REFL_CODE}/2020/snr/p103/p1031050.20.snr99"
    )


#def test_geoidCorrection():
#    assert (
#        geoidCorrection(48.54619475, -123.00761051)
#        == -20.85
#    )





import numpy as np
import os

import matplotlib.pyplot as plt
from matplotlib.figure import Figure


import gnssrefl.gps as g
import gnssrefl.refl_zones as rz

def pickup_files_nyquist(station,recv,obsfile,constel,e1,e2,reqsamplerate,hires_figs):
    """

    Parameters
    ----------
    station : str
        lowercase four character station name
    recv : numpy array
         Cartesian coordinates of station (m)
    obsfile : str
        name of orbit file
    constel : int
        requested constellation (1-4)
    e1 : float
        min elevation angle (deg)
    e2 : float
        max elevation angle (deg)
    reqsamplerate : float
        sample rate of receiver
    hires_figs : bool
        whether you want eps instead of png 

    """
    # for rising and setting arcs
    azlist = np.empty(shape=[0, 3])
    # max number of satellites in each constellation
    # blank,gps, glonass, galileo, beidou
    LV = [0,32,26,36,46]

    # these are in degrees and meters
    lat, lon, nelev = g.xyz2llhd(recv)
    # unit vectors for up, East, and North
    u, East,North = g.up(np.pi*lat/180,np.pi*lon/180)

    f = read_the_orbits(obsfile,constel)

#   examine all satellites - allN will have the nyquists in a list
    allN = np.empty(shape=[0, 2])
    e5 = 5.0
    lv = LV[constel] + 1
    for prn in range(1,lv):
        azlist = np.empty(shape=[0, 3])
        # find the orbit data for that prn number
        newf = f[ (f[:,0] == prn) ]
        if len(newf) > 0:
        # use this to get rising and setting first
            tvsave=rz.calcAzEl_newish(prn, newf,recv,u,East,North)
            nr,nc=tvsave.shape
            #print('shape of tvsave', nr,nc)
            el = tvsave[:,1] - e5
            for j in range(0,nr-1):
                az = round(tvsave[j,2],2)
                newl = [az, prn, e5]
                if ( (el[j] > 0) & (el[j+1] < 0) ) :
                    azlist = np.append(azlist, [newl], axis=0)
                if ( (el[j] < 0) & (el[j+1] > 0) ) :
                    azlist = np.append(azlist, [newl], axis=0)
            # so now you have a list of azimuths
            nrr,ncc = azlist.shape

            # for each rising or setting arc
            for k in range(0,nrr):
                azriseset = azlist[k,0]
                elev = tvsave[:,1]; azims = tvsave[:,2]; t = tvsave[:,3]
                # figure out the nyquist
                nq=rz.nyquist_simple(t,elev,azims, e1,e2,azriseset,reqsamplerate)
                # and then save it
                if not np.isnan(nq):
                    newl=[float(azriseset), float(nq)]
                    allN = np.append(allN, [newl], axis=0);
    nm = ['','GPS','GLONASS','GALILEO','BEIDOU']
    info = str(reqsamplerate) + ' sec sample rate/elev angles '  + str(e1) + '-' + str(e2) + ' ' + nm[constel]

    ny_plot(station,allN,info,hires_figs)


def ny_plot(station,allN, info,hires_figs):
    """
    allN is a numpy array of azimuth(deg)/nyquist(m)
    info is only needed for the title

    Parameters
    ----------
    station : str
        4 char station name

    allN : numpy array ?
        azimuth and nyquist answers
    info : str
        information for the title
    hires_figs : bool
        whether you want eps instead of png

    Returns
    -------
    pngfile : str
        name of plot file

    """
    # wavelengths in meters, being lazy
    l1 = 0.19029360
    l2 = 0.24421
    l5 = 0.254828048
    # defaults
    plot_l2 = True
    plot_l5 = True
    if 'GALILEO' in info:
        plot_l2 = False
    if 'GLONASS' in info:
        plot_l5 = False
    if 'BEIDOU' in info:
        # the beidou signal names are all messed up
        # but that is how they are stored in the RINEX files...
        plot_l5 = False

    #fig = Figure(figsize=(8,4),dpi=360)
    #fig,ax = fig.subplots()
    fig, ax = plt.subplots(figsize=(10,6))
    ax.plot(allN[:,0],allN[:,1],'bo',label='L1')
    if plot_l2:
        ax.plot(allN[:,0],l2*allN[:,1]/l1,'ro',label='L2')
    if plot_l5: 
        ax.plot(allN[:,0],l5*allN[:,1]/l1,'co',label='L5')
    ax.set_title(station + ': Maximum Resolvable RH /' + info, fontsize=14)
    ax.grid()
    ax.set_xlabel('Azimuth (deg)')
    ax.set_ylabel('meters')
    ax.set_xlim(0,360)
    ax.legend(loc="best")

    # checks that directory exists
    g.checkFiles(station, '')

    xdir = os.environ['REFL_CODE'] + '/Files/' + station + '/'


    # a little backwards ... 
    if hires_figs : 
        pngfile = xdir + station + '_maxRH.eps'
        fig.savefig(pngfile, format="eps")
    else:
        pngfile = xdir + station + '_maxRH.png'
        fig.savefig(pngfile, format="png")
    plt.show()
    print('Plot file stored in: ', pngfile)

    txtfile = xdir + station + '_maxRH.txt'
    print(txtfile)
    nr,nc = allN.shape
    N = len(allN)
    fout = open(txtfile, 'w+')
    fout.write('{0:s}  {1:s} \n'.format('%', station + ' ' + info ))
    fout.write('{0:s}  {1:s} \n'.format('%', 'Maximum Resolvable Reflector Height' ))
    for i in range(0,N):
        fout.write('{0:7.2f}  {1:7.2f} \n'.format(allN[i,0], allN[i,1]))
    fout.close()
    print('Writing txtfile to ', txtfile)

def read_the_orbits(obsfile,constel):
    """
    Parameters
    ----------
    obsfile : str
        name of the orbit file to be read
    constel : int
        which constellation (1-4), 1 for gps, 2 for glonass etc
    """
    f = np.genfromtxt(obsfile,comments='%')
    if (constel == 4):
        #print('found beidou')
        i = (f[:,0] < 38) | (f[:,0] > 40)
        f=f[i,:]
    return f




import argparse
import numpy as np
import sys

import gnssrefl.gps as g
import gnssrefl.phase_functions as qp
from gnssrefl.utils import str2bool


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("doy", help="doy", type=int)
    parser.add_argument("-snr", default=None, help="snr file ending", type=int)
    parser.add_argument("-fr", default=None, help="frequency (use all if you want both 1 and 20)", type=str)
    parser.add_argument("-doy_end", "-doy_end", default=None, type=int, help="doy end")
    parser.add_argument("-year_end", "-year_end", default=None, type=int, help="year end")
    parser.add_argument("-e1", default=None, type=float),
    parser.add_argument("-e2", default=None, type=float),
    parser.add_argument("-plt", default=None, type=str, help="plots come to the screen - which you do not want!")
    parser.add_argument("-screenstats", default=None, type=str, help="stats come to the screen")
    parser.add_argument("-gzip", default=None, type=str, help="gzip SNR files after use, default is True" )

    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['plt', 'screenstats', 'gzip']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def quickphase(station: str, year: int, doy: int, year_end: int = None, doy_end: int = None, snr: int = 66, 
        fr: str = '20', e1: float = 5, e2: float = 30, plt: bool = False, screenstats: bool = False, gzip: bool = True):
    """
    quickphase computes phase, which are subquently used in vwc. The command line call is phase
    (which maybe we should change).
    
    Examples
    --------
    phase p038 2021 4
        analyzes data for year 2021 and day of year 4

    phase p038 2021 1 -doy_end 365 
        analyzes data for the whole year

    Parameters
    ----------
    station: str
        4 character ID of the station.
    year: int
        full Year to evaluate.
    doy: int
        day of year to evaluate.
    year_end: int, optional
        year to end analysis. Using this option will create a range from year-year_end.
        Default is None.
    doy_end: int, optional
        Day of year to end analysis. Using this option will create a range of doy-doy_end.
        If also using year_end, then this will be the day to end analysis in the year_end requested.
        Default is None.
    snr : int, optional
        SNR format. This tells the code what elevation angles are in the SNR file
        value options:

            66 (default) : data with elevation angles less than 30 degrees

            99 : data with elevation angles between 5 and 30 degrees

            88 : data with all elevation angles 

            50 : data with elevation angles less than 10 degrees

    fr : str, optional
        GNSS frequency. Currently only supports L2C. Default is 20 (l2c)
    e1 : float, optional
        Elevation angle lower limit in degrees for the LSP. default is 5
    e2: float, optional
        Elevation angle upper limit in degrees for the LSP. default is 30
    plt: bool, optional
        Whether to plot results. Default is False
    screenstats: bool, optional
        Whether to print stats to the screen. Default is False
    gzip : bool, optional
        gzip the SNR file after use.  Default is True

    Returns
    -------
    Saves a file for each day in the doy-doy_end range: $REFL_CODE/<year>/phase/<station>/<doy>.txt

    columns in files:
        year doy hour phase nv azimuth sat ampl emin emax delT aprioriRH freq estRH pk2noise LSPAmp

    """

    compute_lsp = True # used to be an optional input
    if len(station) != 4:
        print('Station name must be four characters long. Exiting.')
        sys.exit()

    if fr == 'all':
        fr_list = [1, 20]
        ex = qp.apriori_file_exist(station,20)
        if (not ex):
            print('No apriori RH file exists. Run vwc_input')
            sys.exit()
    else:
        fr_list = [int(fr)]
        ex = qp.apriori_file_exist(station,int(fr))
        if (not ex):
            print('No apriori RH file exists. Run vwc_input')
            sys.exit()


    # in case you want to analyze multiple days of data
    if not doy_end:
        doy_end = doy

    exitS = g.check_inputs(station, year, doy, snr)

    if exitS:
        sys.exit()

    g.result_directories(station, year, '')

    pele = [5, 30]  # polynomial fit limits  for direct signal

    # TODO maybe instead of specific doy we can do only year and pick up all those files just like the other parts?
    if year_end:
        year_range = np.arange(year, year_end+1)
        for y in np.arange(year, year_end+1):
            # If first year in multi-year range, then start on doy requested and finish year.
            if y == year_range[0]:
                date_range = np.arange(doy, 366)
            # If last year in multi-year range, then start on doy 1 finish on doy_end.
            elif y == year_range[-1]:
                date_range = np.arange(1, doy_end+1)
            # If year within multi-year range then do whole year start to finish.
            else:
                date_range = np.arange(1, 366)

            for d in date_range:
                print('Analyzing year/day of year ' + str(y) + '/' + str(d))
                qp.phase_tracks(station, y, d, snr, fr_list, e1, e2, pele, plt, screenstats, compute_lsp, gzip)
    else:
        for d in np.arange(doy, doy_end + 1):
            qp.phase_tracks(station, year, d, snr, fr_list, e1, e2, pele, plt, screenstats, compute_lsp, gzip)


def main():
    args = parse_arguments()
    quickphase(**args)


if __name__ == "__main__":
    main()



import warnings
import argparse
import os
import subprocess
import sys
import wget
import multiprocessing
from functools import partial

#import gnssrefl.gnssir as guts
import gnssrefl.gnssir_v2 as guts2
import gnssrefl.gps as g

from gnssrefl.utils import str2bool


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name (4 ch)", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("doy", help="day of year", type=int)

    # optional inputs
    parser.add_argument("-snr", default=None, help="snr file ending, default is 66", type=int)
    parser.add_argument("-plt", default=None, help="plt to screen (True or False)", type=str)
    parser.add_argument("-fr", default=None, type=int, help="Frequency, 1 for GPS L1, 101 for Glonass L1 etc")
    parser.add_argument("-ampl", default=None, type=float, help="Min spectral amplitude ")
    parser.add_argument("-sat", default=None, type=int, help="Only look at this satellite")
    parser.add_argument("-doy_end", default=None, type=int, help="doy end")
    parser.add_argument("-year_end", default=None, type=int, help="year end")
    parser.add_argument("-azim1", default=None, type=int, help="lower limit azimuth (deg)")
    parser.add_argument("-azim2", default=None, type=int, help="upper limit azimuth (deg)")
    parser.add_argument("-nooverwrite", default=None, type=str, help="default is False, i.e. you will overwrite")
    parser.add_argument("-extension", type=str, help="extension for result file, useful for testing strategies")
    parser.add_argument("-compress", default=None, type=str, help="Boolean, xz compress SNR files after use")
    parser.add_argument("-gzip", default=None, type=str, help="Boolean, gzip SNR files after use. Default is True")
    parser.add_argument("-screenstats", default=None, type=str, help="Boolean, some stats printed to screen(default is False)")
    parser.add_argument("-delTmax", default=None, type=int, help="Allowed satellite arc length (minutes)")
    parser.add_argument("-e1", default=None, type=float, help="min elev angle (deg)")
    parser.add_argument("-e2", default=None, type=float, help="max elev angle (deg)")
    parser.add_argument("-mmdd", default=None, type=str, help="Boolean, add columns for month,day,hour,minute")
    parser.add_argument("-dec", default=1, type=int, help="decimate SNR file to this sampling rate before computing periodograms")
    parser.add_argument("-newarcs", default=None, type=str, help="This no longer has any meaning")
    parser.add_argument("-par", default=None, help="int, set number of parallel processes to run when processing data", type=int)


    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['plt', 'screenstats', 'nooverwrite', 'compress', 'screenstats', 'mmdd','gzip','newarcs']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def gnssir(station: str, year: int, doy: int, snr: int = 66, plt: bool = False, fr: int = None, 
        ampl: float = None, sat: int = None, doy_end: int = None, year_end: int = None, azim1: int = 0, 
        azim2: int = 360, nooverwrite: bool = False, extension: str = '', compress: bool = False, 
        screenstats: bool = False, delTmax: int = None, e1: float = None, e2: float = None, 
        mmdd: bool = False, gzip: bool = True, dec : int = 1, newarcs : bool = True, par : int = None):

    """
    gnssir is the main driver for estimating reflector heights. The user is required to 
    have set up an analysis strategy using gnssir_input. 

        
    Examples
    --------
    gnssir p041 2021 15 
        analyzes the data for station p041, year 2021 and day of year 15.
    gnssir p041 2021 15  -snr 99
        uses SNR files with a 99 suffix
    gnssir p041 2021 15  -plt T
        plots of SNR data and periodograms come to the screen. Each frequency gets its own plot.
    gnssir p041 2021 15  -screenstats T
        sends debugging information to the screen
    gnssir p041 2021 15  -nooverwrite T 
        only runs gnssir if there isn't a previous solution
    gnssir p041 2021 15  -extension strategy1
        runs gnssir using json file called p041.strategy1.json
    gnssir p041 2021 15  -doy_end 20 
        Analyzes data from day of year 15 to day of year 20
    gnssir p041 2021 15 -dec 5
        before computing periodograms, decimates the SNR file contents to 5 seconds
    gnssir p041 2021 15 -gzip T
        gzips the SNR file after you run the code. Big space saver (now the default)

    Parameters
    ----------
    station : str
        lowercase 4 character ID of the station
    year : int
        full Year
    doy : integer
        Day of year
    snr : int, optional
        SNR format. This tells the code what elevation angles to save data for. Input is the snr file ending.
        Value options:

            66 (default) : saves all data with elevation angles less than 30 degress

            99 : saves all data with elevation angles between 5 and 30 degrees

            88 : saves all data 

            50 : saves all data with elevation angles less than 10 degrees

    plt : bool, optional
        Send plots to screen or not. Default is False.
    fr : int, optional
        GNSS frequency. Value options:

            1,2,20,5 : GPS L1, L2, L2C, L5

            101,102 : GLONASS L1, L2

            201, 205,206,207,208 : GALILEO E1, E5a,E6,E5b,E5

            302,306,307 : BEIDOU B1, B3, B2 (not sure we do 307)

    ampl : float, optional
        minimum spectral peak amplitude. default is None
    sat : int, optional
        satellite number to only look at that single satellite. default is None.
    doy_end : int, optional
        end day of year. This is to create a range from doy to doy_end of days.
        If year_end parameter is used - then day_end will end in the day of the year_end.
        Default is None. (meaning only a single day using the doy parameter)
    year_end : int, optional
        end year. This is to create a range from year to year_end to get the snr files for more than one year.
        doy_end will be for year_end. Default is None.
    azim1 : int, optional
        lower limit azimuth.
        If the azimuth angles are changed in the json (using 'azval' key) and not here, then the json overrides these.
        If changed here, then it overrides what you requested in the json. default is 0.
    azim2 : int, optional
        upper limit azimuth.
        If the azimuth angles are changed in the json (using 'azval' key) and not changed here, then the json overrides these.
        If changed here, then it overrides what you requested in the json. default is 360.
    nooverwrite : bool, optional
        Use to overwrite lomb scargle result files or not.
        Default is False, i.e., it will overwrite.
    extension : string, optional
        extension for result file, useful for testing strategies. default is empty string
    compress : boolean, optional
        xz compress SNR files after use. default is False.
    screenstats : bool, optional
        whether to print stats to the screen or not. default is True.
    delTmax : int, optional
        maximum satellite arc length in minutes. found in the json
    e1 : float, optional
        use to override the minimum elevation angle.
    e2 : float, optional
        use to override the maximum elevation angle.
    mmdd : boolean, optional
        adds columns in results for month, day, hour, and minute. default is False.
    gzip : boolean, optional
        gzip compress SNR files after use. default is True (as of 2023 Sep 17).
    dec : int, optional
        decimate SNR file to this sampling period before the 
        periodograms are computed. 1 sec is default (i.e. no decimating)
    newarcs : bool, optional
        this input no longer has any meaning 
    par : int, optional
        default is 1 process. can increase it to allow for parallel processing of data retrieval. 
        only useful when processing more than 1 year at once using -year_end argument

    """

#   make sure environment variables exist.  set to current directory if not
    g.check_environ_variables()

    exitS = g.check_inputs(station, year, doy, snr)

    if exitS:
        sys.exit()

    lsp = guts2.read_json_file(station, extension)
    # now check the overrides to the json instructions

    # requiring people use the new code
    if 'azval2' not in lsp:
        print('An azval2 variable was not found in your json input file. Fix your json ')
        print('and/or use gnssir_input to make a new one. Exiting')
        sys.exit()

    #if newarcs:
    #else:
    #if 'azval' in lsp:
    #    print('An azval variable was found in your json. I will use the old way of defining arcs.')
    #else:
    #    print('You chose the old way of setting arcs, but you do not have valid input in your json.')
    #    print('I recommend you use gnssir_input to create your json file. Azimuth regions are no ')
    #    print('longer required to be regions less than 100 degrees. Exiting.')
    #        sys.exit()


    # plt is False unless user changes
    lsp['plt_screen'] = plt

    if delTmax is not None:
        lsp['delTmax'] = delTmax

    if ((lsp['maxH'] - lsp['minH']) < 5):
        print('Requested reflector heights (', lsp['minH'], ',', lsp['maxH'], ') are too close together. Exiting.')
        print('They must be at least 5 meters apart - and preferably further than that.')
        sys.exit()

    # compress is False unless user changes
    lsp['wantCompression'] = compress

    # screenstats is True unless user changes
    lsp['screenstats'] = screenstats

    # added this for people that have 1 sec files that do not need this resolution
    # decimation is a bit slow but way faster than doing a gazillion periodograms with 
    # 1 sec data

    lsp['dec'] = dec

    # in case you want to analyze multiple days of data
    if doy_end is None:
        doy_end = doy

    # mmdd is False unless user changes
    add_mmddhhss = mmdd


    # in case you want to analyze only data within one year
    year_st = year  # renaming for confusing variable naming when dealing with year calculations below
    if year_end is None:
        year_end = year_st

    # default will be to overwrite
    #if nooverwrite is None:
    lsp['nooverwrite'] = nooverwrite
    #else:
    #    lsp['overwriteResults'] = False

    if e1 is not None:
        lsp['e1'] = e1
    if e2 is not None:
        lsp['e2'] = e2

    # in case you want to look at a restricted azimuth range from the command line
    setA = 0
    # if azim1 is not default (has been changed)
    if azim1 != 0:
        setA = 1

    # if azim2 is not default (has been changed)
    if azim2 != 360:
        setA = setA + 1

    # TODO figure out the goal here
    # this only sets the new azim values only if bother azim1 and azim2 changed?
    if setA == 2:
        lsp['azval'] = [azim1,  azim2]

    # this is for when you want to run the code with just a single frequency, i.e. input at the console
    # rather than using the input restrictions
    if fr is not None:
        lsp['freqs'] = [fr]
    if ampl is not None:
        # this is not elegant - but allows people to set ampl on the command line
        # but use the frequency list from their json ...  which i think has max of 12
        # but use 14 to be sure
        lsp['reqAmp'] = [ampl for i in range(14)]

    if sat is not None:
        lsp['onesat'] = [sat]

    lsp['mmdd'] = add_mmddhhss
    # added 2022apr15
    lsp['gzip'] = gzip

    # if refraction model is not assigned, set it to 1
    if 'refr_model' not in lsp.keys():
        lsp['refr_model'] = 1

    xdir = str(os.environ['REFL_CODE'])
    picklefile = 'gpt_1wA.pickle'
    pname = xdir + '/input/' + picklefile

    #    print('refraction file exists')
    if not os.path.isfile(pname):
        local_copy = 'gnssrefl/' + picklefile
        if os.path.isfile(local_copy):
            print('found local copy of refraction file')
            subprocess.call(['cp', '-f', local_copy, xdir + '/input/'])
        else:
            print('download and move refraction file')
            url='https://github.com/kristinemlarson/gnssrefl/raw/master/gnssrefl/gpt_1wA.pickle'
            wget.download(url, picklefile)
            subprocess.call(['mv', '-f', picklefile, xdir + '/input/'])

    args = {'station': station.lower(), 'year': year, 'doy': doy, 'snr_type': snr, 'extension': extension, 'lsp': lsp}

    print(lsp['pele'], ' direct signal elevation angle limits')
    print(lsp['e1'], lsp['e2'], ' min and max elevation angles')
    # added this because ellist is a new option and was not necessarily created in old json files
    if 'ellist' not in lsp:
        print('did not find ellist')
        if float(lsp['e1']) < float(lsp['pele'][0]):
            print('emin is smaller than the minimum eangle (pele) used for direct signal removed.')
            print('This is Forbidden. Fix the records set in the json created by gnssir_analysis')
            sys.exit()
    else:
        if float(lsp['e1']) < float(lsp['pele'][0]):
            print('Your requested emin is lower than the minimum elevation angle used ')
            print('for direct signal removal. This is stored in the variable pele.')
            print('This is Forbidden. Fix the pele records set in the json created by gnssir_analysis')
            sys.exit()
        if float(lsp['e2']) > float(lsp['pele'][1]):
            print('Your requested emax is higher than the maximum elevation angle used ')
            print('for direct signal removal. This is stored in the variable pele.')
            print('This is Forbidden. Fix the pele records set in the json created by gnssir_analysis')
            sys.exit()

    # should make sure there are directories for the results ... 
    g.checkFiles(station.lower(), extension)

    year_list = list(range(year_st, year_end+1))
    # changed to better describe year and doy start/end

    print('Requested frequencies ', lsp['freqs'])

    additional_args = { "year_end": year_end, "year_st": year_st, "doy": doy, "doy_end": doy_end, "args": args }

    if not par: 
        for year in year_list:
            process_year(year, **additional_args)
    else:
        pool = multiprocessing.Pool(processes=par) 
        partial_process_year = partial(process_year, **additional_args)

        pool.map(partial_process_year, year_list)
        pool.close()
        pool.join()


def process_year(year, year_end, year_st, doy, doy_end, args):
    """
    Code that does the processing for a specific year. Refactored to seperate function to allow for parallel processes

    Parameters
    ----------
    year : int
        full Year
    year_end : int
        end year. This is to create a range from year to year_end to get the snr files for more than one year.
        doy_end will be for year_end. Default is None.
    year_st: int 
        TODO what is this?    
    doy : integer
        Day of year
    doy_end : int
        end day of year. This is to create a range from doy to doy_end of days.
        If year_end parameter is used - then day_end will end in the day of the year_end.
        Default is None. (meaning only a single day using the doy parameter)
    args : dict
        arguments passed into gnssir through terminal
    """
    # edits made 2021Sep10 by Makan karegar
    if year != year_end:
        doy_en = 366
    else:
        doy_en = doy_end

    # edits made 2021Sep10 by Makan karegar
    if year == year_st:
        doy_list = list(range(doy, doy_en+1))
    else:
        doy_list = list(range(1, doy_en+1))

    args['year'] = year
    for doy in doy_list:
        args['doy'] = doy
        try:
            guts2.gnssir_guts_v2(**args)
        except:
            warnings.warn(f'error processing {year} {doy}');                
        #else:
        #    print('You are trying to use the Old Way of Selecting Arcs.')
        #    print('This is no longer supported. Update your code')
        #guts.gnssir_guts(**args)


def main():
    args = parse_arguments()
    gnssir(**args)


if __name__ == "__main__":
    main()





import scipy.io
from scipy.interpolate import interp2d
import os

class EGM96geoid:
    """ Class for EGM96 geoid corrections

    :example:

    >>> egm = EGM06geoid()
    >>> egm.heights(lat=10, lon=30)
    -5.32
    """

    def __init__(self):
        self.geoid = {}

        # Loads the struct called: geoid  
        #
        #    'grid' = geoid height (m) in .25deg steps
        #    'lats' = row vector of latitudes, deg (-92,92)
        #    'lons' = row vector of longitudes, deg (-2,362)

        xdir = os.environ['REFL_CODE']
        # this may go better in the input directory - but using Files for now
        egmfile = xdir + '/Files/' + 'EGM96geoidDATA.mat'
        matdata = scipy.io.loadmat(egmfile)
        names = matdata['geoid'].dtype.names

        for name, arr in zip(names, matdata['geoid'].item()):
            self.geoid[name] = arr.astype('float')

        # create interpolating function from grid data
        self.h = interp2d(self.geoid['lons'][0,:], self.geoid['lats'][0,:], self.geoid['grid'], kind='cubic', bounds_error=True)

    def height(self, lat: float, lon: float):
        # One set of lat lon in, one height out

        lon = lon % 360 # Make sure 0 <= lon < 360

        # Fix geoid height at hundreth of a meter
        return round(self.h(lon, lat)[0], 2)

    



import argparse
import json
import os
import subprocess
import csv
import sys
from pathlib import Path
import requests
import numpy as np
import collections

import gnssrefl.gps as g
from gnssrefl.utils import str2bool
from datetime import datetime

from earthscope_sdk.auth.device_code_flow import DeviceCodeFlowSimple


def parse_arguments():
    # user inputs the observation file information
    parser = argparse.ArgumentParser()
    # required arguments
    parser.add_argument("station", help="station (lowercase)", type=str)
    # optional inputs
    parser.add_argument(
        "-lat",
        help="Latitude (deg), if station not in database",
        type=float,
        default=None,
    )
    parser.add_argument(
        "-lon",
        help="Longitude (deg), if station not in database",
        type=float,
        default=None,
    )
    parser.add_argument(
        "-height",
        help="Ellipsoidal height (m) if station not in database",
        type=float,
        default=None,
    )
    parser.add_argument(
        "-read_offset",
        default=None,
        type=str,
        help="append meta from GAGE offset file",
    )
    parser.add_argument(
        "-man_input", default=None, type=str, help="append meta from manual input"
    )
    parser.add_argument(
        "-overwrite", default=None, type=str, help="create new meta file"
    )

    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = [
        "read_offset",
        "man_input",
        "overwrite",
    ]
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def make_meta(
    station: str,
    lat: float = None,
    lon: float = None,
    height: float = None,
    man_input: bool = True,
    read_offset: bool = False,
    overwrite: bool = False,
):
    """
    Make a json file that includes equipment metadata information.
    It saves your inputs to a json file saved in REFL_CODE/input/<station>_meta.json.

    If station is in the UNR database, those lat/lon/ht values are used. You may override those values
    with the optional inputs.

    The default is for the user to enter these values; multiple calls will append entries to the metadata array,
    unless the user sets overwrite to True.
    The user can attempt to extract some of this information from the GAGE offset file.
    GAGE file used is https://data.unavco.org/archive/gnss/products/offset/cwu.kalts_nam14.off (requires ES auth)
    [A caveat: this file is comprehensive for antennas changed by stations included in GAGE processing (n=~3k).
    Receivers are included, but incomplete.]

    Examples
    --------
    make_meta p038
        makes json meta file for p038; uses UNR coords and will request user manually populate meta info.
        If meta json already exists for p038, will append to existing file.

    make_meta test -lat 39.7417583 -lon -105.0706972 -height 1655
        makes json meta file for test; uses manually input coords and will request ueser manually populate meta info

    make_meta p038 -man_input False -read_offset True -overwrite True
        makes json meta file for p038; uses UNR coords, will not request user manually populate meta info but will
        extract available meta info from GAGE offset file.
        Will overwrite any existing meta json file for p038

    Parameters
    ----------
    station : str
        4 character station ID.

    lat : float, optional
        latitude in deg

    lon : float, optional
        longitude in deg

    height : float, optional
        ellipsoidal height in m

    read_offset : bool, optional
        set to True to parse GAGE offset file. default is False.

    man_input : bool, optional
        set to True to manually input equipment metadata. default is True.

    overwrite : bool, optional
        set to True to overwrite existing metadata file. default is False.

    """

    # make sure environment variables exist
    g.check_environ_variables()

    # meta json object path
    xdir = os.environ["REFL_CODE"]
    outputdir = xdir + "/input"
    if not os.path.isdir(outputdir):
        subprocess.call(["mkdir", outputdir])
    outputfile = outputdir + "/" + station + "_meta.json"

    # If no overwrite and a file already exists, read in existing meta json
    if (overwrite is False) and (os.path.isfile(outputfile)):
        with open(outputfile) as json_file:
            comp_dict = json.load(json_file)
            meta_dict = comp_dict["meta"]

    else:  # initialize empty meta_dict
        comp_dict = get_coords(station, lat, lon, height)
        meta_dict = {}

    # read in gage metadata file
    if read_offset:
        meta_dict = check_offsets(station, meta_dict)

    if man_input:
        meta_dict = meta_man_input(meta_dict)

    # sort by date
    meta_dict = collections.OrderedDict(sorted(meta_dict.items()))

    # add metad to complete dictionary
    comp_dict["meta"] = meta_dict
    
    print('writing meta json out to:', outputfile)
    with open(outputfile, "w+") as outfile:
        json.dump(comp_dict, outfile, indent=3)

def get_coords(station, lat, lon, height):
    """
    initializes metadata dictionary with lat lon ht keys, either from UNR database (default) or user entered

    Parameters
    ----------
    station : str
        4 character station ID.

    lat : float, optional, default is None
        latitude in deg

    lon : float, optional, default is None
        longitude in deg

    height : float, optional, default is None
        ellipsoidal height in m

    Returns
    ----------
    comp_dict : dict
        dictionary of metadata; keys 'lat','lon','ht' 'meta'.

    """

    if (lat is None) & (lon is None):
        # check the station coordinates in our database from the station name
        lat, lon, height = g.queryUNR_modern(station)
        if (lat == 0) and (lon == 0):
            print(
                "Manually input coords using -lat -lon -ht args for make_meta. \n Exiting."
            )
            sys.exit()
        else:
            print("Using inputs:", lat, lon, height)
    else:
        print("Using inputs:", lat, lon, height)

    comp_dict = {
        "station": station,
        "lat": "{:.4f}".format(lat),
        "lon": "{:.4f}".format(lon),
        "ht": "{:.4f}".format(height),
        "meta": {},
    }
    return comp_dict


def meta_man_input(meta_dict):
    """
    allows user to manually enter metadata information [Rx, Antenna, Dome, FW] at a given YYYY-mm-dd

    Parameters
    ----------
    meta_dict : dict
        dictionary of metadata; keys 'dates','current','previous'.

    Returns
    ----------
    meta_dict : dict
        dictionary of metadata; keys 'dates','current','previous'.
    """

    date_str = input(
        "You have chosen to manually enter a metadata state. \n Enter the date: (YYYY-mm-dd): "
    )
    # Check if datestring entered is a valid string
    date_valid = False
    while date_valid is False:
        try:
            date_valid = bool(datetime.strptime(date_str, "%Y-%m-%d"))

        except ValueError:
            date_str = input(
                "%s is not in the correct input date format.  Please re-enter the date: (YYYY-mm-dd): "
                % (date_str)
            )

    receiver = input(" Enter the receiver brand and model: ")
    antenna = input("Enter the antenna model: ")
    dome = input("Enter the radome 4 digit type or * if unknown: ")
    fw = input("Enter the receiver firmware or * if unknown: ")

    meta_dict[date_str] = {
        "state": receiver + " " + antenna + " " + dome + " " + fw,
    }
    return meta_dict


def get_es_sdk_headers():
    """
    checks for earthscope-sdk authentication token and refreshes
    repurposed from gnssrefl/kelly.py

    Returns
    ----------
    header : dict
        dictionary of es-sdk token

    """

    token_path = "./"
    device_flow = DeviceCodeFlowSimple(Path(token_path))

    print("Seeking permission from Earthscope to use their archive")
    try:
        # get access token from local path
        device_flow.get_access_token_refresh_if_necessary()
    except:
        # if no token was found locally, do the device code flow
        device_flow.do_flow()

    token = device_flow.access_token

    headers = {}
    headers["authorization"] = "Bearer " + token
    return headers


def check_offsets(station, meta_dict):
    """
    check GAGE processing offset file for sources of possible gnssrefl timeseries offsets
    currently only relevant for ~3k stations processed by GAGE

    Parameters
    ----------
    station : str
        4 character station ID.

    meta_dict : dict
        dictionary of metadata; keys 'dates','current','previous'.

    Returns
    ----------
    meta_dict : dict
        dictionary of metadata; keys 'dates','current','previous'.
    """
    offset_meta_dates = []
    offset_meta_dict_list = []

    url = "https://data.unavco.org/archive/gnss/products/offset/cwu.kalts_nam14.off"
    print("reading GAGE offset file at %s" % url)
    headers = get_es_sdk_headers()
    with requests.Session() as s:
        response = s.get(url, stream=True, headers=headers)
        decode_response = response.content.decode("utf-8")
        cr = list(csv.reader(decode_response.splitlines()))
        for row in cr:
            if (row[0][1:5] == station.upper()) and (len(row[0].split()) > 14):
                # DATE
                year = int(row[0].split()[1])
                month = int(row[0].split()[2])
                day = int(row[0].split()[3])

                offset_meta_dates.append(
                    datetime(year=year, day=day, month=month).strftime("%Y-%m-%d")
                )

                # Description
                ofs = row[0].split()[11]
                evt = row[0].split()[14]
                # todo handle (rare) cases when order in file is swapped
                if evt == "AN":
                    tmp_dict = {}
                    rx_array = np.empty(2, dtype="S15")
                    for i, state in enumerate(["state", "prior"]):
                        receiver = rx_array[i] = (
                            row[0].split()[24 - (i * 3)]
                            + " "
                            + row[0].split()[24 - (i * 3) + 1]
                        )
                        antenna = row[0].split()[19 - (i * 2)]
                        dome = row[0].split()[29 - (i * 2)]
                        tmp_dict[state] = "%s %s %s *" % (
                            receiver,
                            antenna,
                            dome,
                        )
                    swap = "ANT"
                    if rx_array[0] != rx_array[1]:
                        swap = "ANT/RX"
                    tmp_dict["swap"] = swap
                offset_meta_dict_list.append(tmp_dict.copy())

        offset_meta_dict = dict(zip(offset_meta_dates, offset_meta_dict_list))
        meta_dict = meta_dict | offset_meta_dict  # .copy()

    if len(offset_meta_dates) == 0:
        print("%s does not have an entry in GAGE offset file" % station)
    return meta_dict


def main():
    args = parse_arguments()
    make_meta(**args)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
"""
import argparse
import os
import subprocess 
import sys

import gnssrefl.gps as g
import gnssrefl.rinex2snr as r

def main():
    """

    Creates SNR file from RINEX 3 file that is stored locally
    Requires the gfzrnx executable to be available.

    Parameters
    ----------
    rinex3 : str
        name of RINEX3 file
    orb : str
        optional orbit choice.  default is gbm
    snr : int
        snr file choice. default is 66

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("rinex3", help="rinex3 filename", type=str)
    parser.add_argument("-orb", help="orbit choice", default='gbm',type=str)
    parser.add_argument("-dec", help="decimation", default=None,type=int)
    parser.add_argument("-snr", help="SNR file ending (default is 66)", default=None,type=str)
    parser.add_argument("-earthscope_fail", help="boolean(T or True), earthscope created illegal filenames", default=None,type=str)


    args = parser.parse_args()
    if args.snr is None:
        snr = '66'
    else:
        snr = int(args.snr)

    if args.dec is None:
        dec_rate = 0; 
    else:
        dec_rate = args.dec

    if (args.earthscope_fail == 'T') or (args.earthscope_fail == 'True'):
        # earthscope felt the need to distribute illegal RINEX files.
        # this is an attempt to help those people impacted by this activity.
        illegal_file = args.rinex3
        print('Earthscope using an illegal filename for RINEX 3 data:', illegal_file)

        station = illegal_file[0:4].lower()
        STATION = illegal_file[0:4].upper() + '00ANT'
        cdoy = illegal_file[4:7]
        year = '20' + illegal_file[9:11]
        idoy = int(cdoy) 
        rinex3 = STATION + '_R_' + year + cdoy + '0000_01D_30S_MO.rnx'
        # move the illegal file to its proper name
        subprocess.call(['mv','-f',illegal_file,rinex3])

    else:

        rinex3 = args.rinex3
        station = rinex3[0:4].lower()
        STATION = rinex3[0:4]
        year = rinex3[12:16]
        cdoy = rinex3[16:19] 
        idoy = int(cdoy)

    #print(station, STATION, year, cdoy)
    # all lowercase in my world
    rinex2 = station + cdoy + '0.' + year[2:4] + 'o'
    orbtype = args.orb 
    if orbtype == 'gps':
        orbtype = 'nav'
    elif orbtype == 'gnss':
        orbtype = 'gbm'
    elif orbtype == 'gps+glo':
        orbtype = 'gbm'

    gexe = g.gfz_version()

    if not os.path.exists(gexe):
        print('Required gfzrnx executable does not exist. Exiting.')
        sys.exit()


    # first step will be to translate to rinex2

    if os.path.isfile(rinex3):
        print('Found version 3 input file ')
        if (rinex3[-2::] == 'gz'):
            print('Must gunzip version 3 rinex')
            subprocess.call(['gunzip', rinex3])
            rinex3 = rinex3[0:-3]
        g.new_rinex3_rinex2(rinex3,rinex2)
    else:
        print('ERROR: your input file does not exist:', rinex3)
        sys.exit()
    if os.path.isfile(rinex2):
        print('found version 2 rinex')
        isnr = 66;  rate = 30; idoy = int(cdoy); iyear = int(year)
        rate = '30'; 
        archive = 'unavco' ; fortran = False; translator = 'hybrid'
        year_list = [iyear]; doy_list = [idoy]; rate = 'low';   nol = True; 
        overwrite = False; srate = 30; mk = False; skipit = 1
        strip = False; stream = 'S'  ; bkg = 'IGS' # many of these are fake values because the file has already been translated to rinex2
        gzip = True
        timeout = 0
        screenstats = False
        # removed fortran and skipit inputs ...  and got rid of the year and doy lists
        # 2024 may 28
        r.run_rinex2snr(station, iyear, idoy, isnr, orbtype, rate,dec_rate,archive,nol,
                overwrite,translator,srate,mk,stream,strip,bkg,screenstats,gzip,timeout)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import argparse
from astropy.time import Time
import datetime
import matplotlib.pyplot as myplt
import numpy as np
import os
import subprocess
import sys


import gnssrefl.quicklib as q
import gnssrefl.gps as g

from gnssrefl.utils import validate_input_datatypes, str2bool

def parse_arguments():
    parser = argparse.ArgumentParser()

    parser.add_argument("filename", help="filename", type=str)
    parser.add_argument("xcol", help="x-axis column number", type=str)
    parser.add_argument("ycol",   help="y-axis column number", type=str)
    parser.add_argument("-errorcol",   help="column for errors for y-axis values", type=int,default=None)
    parser.add_argument("-mjd", help="True/T for when x-values are MJD ", type=str,default=None)
    parser.add_argument("-reverse", help="reverse the y-axis", type=str,default=None)
    parser.add_argument("-ymd", help="True/T for when columns 1-3 are year month day ", type=str,default=None)
    parser.add_argument("-ymdhm", help="True/T for when columns 1-5 are year month day hour minute", type=str,default=None)
    parser.add_argument("-xlabel", type=str, help="optional x-axis label", default=None)
    parser.add_argument("-ylabel", type=str, help="optional y-axis label", default=None)
    parser.add_argument("-symbol", help="plot symbol, e.g. o or -", type=str,default=None)
    parser.add_argument("-title", help="optional title", type=str,default=None)
    parser.add_argument("-outfile", help="optional filename for plot. Must end in png", type=str,default=None)
    parser.add_argument("-xlimits", nargs="*",type=str, help="optional x-axis limits, use yyyyMMdd", default=None)
    parser.add_argument("-ylimits", nargs="*",type=float, help="optional y-axis limits", default=None)
    parser.add_argument("-ydoy", help="True/T for when columns 1-2 are year and doy", type=str,default=None)
    parser.add_argument("-filename2", help="second filename", type=str, default=None)
    parser.add_argument("-freq", help="spec freq, column 11 in a LSP file ", type=int,default=None)
    parser.add_argument("-utc_offset", help="offset from UTC, hours  ", type=int,default=None)
    parser.add_argument("-yoffset", help="offset for y-axis values", type=float,default=None)
    parser.add_argument("-yoffset2", help="offset for y-axis values in file 2", type=float,default=None)
    parser.add_argument("-keepzeros", help="True/T if you want to keep zero y-values (default is to remove)", type=str,default=None)
    parser.add_argument("-sat", help="Only use this satellite (can also input gps, glonass, galileo, and beidou) SNR file", type=str,default=None)
    parser.add_argument("-scale", help="scale factor for first file", type=float,default=None)
    parser.add_argument("-scale2", help="scale factor for file 2", type=float,default=None)
    parser.add_argument("-elimits", nargs="*",type=float, help="optional elevation angle limits for SNR file", default=None)
    parser.add_argument("-azlimits", nargs="*",type=float, help="optional azimuth angle limits for SNR file", default=None)
    parser.add_argument("-plt", help="Set to False/F if you do not want the plot to come to the screen ", type=str,default=None)

    g.print_version_to_screen()

    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['mjd', 'reverse', 'ymdhm', 'ydoy','ymd','keepzeros','plt']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def run_quickplt (filename: str, xcol: str, ycol: str, errorcol: int=None, mjd: bool=False, xlabel: str=None, 
                  ylabel: str=None, symbol: str=None, reverse:bool=False,title:str=None,outfile: str=None,
                  xlimits: str=None, ylimits: float=[], ydoy:bool=False, ymd:bool=False, ymdhm:bool=False, 
                  filename2: str=None, freq:int=None, utc_offset: int=None, yoffset: float=None, 
                  keepzeros: bool=False, sat: str=None,yoffset2: float=None,scale: float=1.0, 
                  scale2: float=1.0, elimits: float=[0], azlimits:float=[0], plt:bool=True):

    """
    quick file plotting using matplotlib

    A png file is saved as temp.png or to your preferred
    filename if outfile is given. In either case, it goes to REFL_CODE/Files

    Allows you to set x and y-axis limits with a title and various axes labels, symbols etc.
    
    Someone could easily update this to include different filetypes (e.g. jpeg)

    I rewrote this recently to take advantage of our boolean argument translator.  Let me know 
    if things have broken or submit a PR.

    To make simple plots of observables in SNR files, the x-axis can be either time or elevation.
    The latter is short for elevation angle. To pick this option set -sat to a specific satellite
    nubmer or a constellation (gps, glonass, etc). You can also set elimits and azlimits for simple
    elevation angle and azimuth angle limits. Only for SNR files, you can send the name of the SNR file
    without the directory, i.e. sc021500.22.snr66 instead of /Users/Files/2022/snr/sc02/sc021500.22.snr66

    You may submit a filename that has been gzipped. The code will checked to see if the gzip version
    is there and gunzip it for you.


    Examples
    --------
    quickplt txtfile 1 16
        would plot column 1 on the x-axis and column 16 on the y-axis

    quickplt sc021500.22.snr66 time L1 -sat gps
        would plot all the GPS L1 SNR data for the given SNR file, with time on the 
        x-axis column 1 on the x-axis and SNR data on the y-axis
        You have to set -sat or it will not work. For a specific satellite number,
        provide that instead of gps. The other allowed x-axis option is elevation which
        is short for elevation angle.

    quickplt txtfile 1 16 -xlabel Time
        would plot column 1 on the x-axis and column 16 on the y-axis
        and add Time on the x-axis label

    quickplt txtfile 1 16 -xlabel "Time (sec)"
        would plot column 1 on the x-axis and column 16 on the y-axis
        and add Time (sec) on the x-axis label, but need quote marks since you 
        have spaces in the x-axis labe.

    quickplt txtfile 1 16 -reverse T
        would plot column 1 on the x-axis and column 16 on the y-axis
        it would reverse the y-axis parameter as you might want if 
        you are ploting RH but want it to have the same sense as a tide gauge.

    quickplt txtfile 1 3 -errorcol 4 
        would plot error bars from column 4 

    quickplt txtfile 1 3 -errorcol 4 -ydoy T
        would plot error bars from column 4  and assume columns 1 and 2 
        are year and day of year

    quickplt txtfile 1 4 -mjd T
        assume column 1 is modified julian day

    quickplt txtfile 1 16 -ylimits 0 2
        would restrict y-axis to be between 0 and 2

    quickplt txtfile 1 16 -outfile myfile.png
        would save png file to $REFL_CODE/Files/myfile.png

    quickplt snrfile 4 7 
        Assuming you submitted a standard SNR file, it would plot the 
        L1 data (column 7) vs time (seconds of the day in column 4). Zeroes would
        be removed, but you can toggle that to keep it.

    quickplt snrfile 2 7 
        Assuming you submitted a standard SNR file, it would plot the 
        L1 data (column 7) vs elevation angle (degrees in column 2). 

    quickplt snrfile 4 8 -sat 25
        Assuming you submitted a standard SNR file, it would plot the 
        data for satellite 25 for L2 data (column 8) vs time (seconds of the day in column 4). 

    quickplt snrfile 4 8 -sat 22
        Assuming you submitted a standard SNR file, it would plot the 
        data for Glonass satellite 22 for L2 data (column 8) vs time (seconds of the day in column 4). 

    Parameters
    ----------
    filename : str
        name of file to be plotted 
    xcol : str
        column number in the file for the x-axis parameter
        for snrfiles, you can say time or elevation
    ycol : str
        column number in the file for the y-axis parameter
        for snrfiles, you can say L1, L2, or L5
    errorcol : int, optional
        column number for the error bars
    mjd : bool, optional
        code will convert MJD to datetime (for xcol) 
    xlabel : str, optional
        label for x-axis 
    ylabel : str
        label for y-axis 
    symbol : str, optional
        prescribe the marker used in the plot . It can include the color, i.e.
        'b.' or 'b^'
    reverse : bool, optional
        to reverse y-axis limits
    title : str, optional
        title for plot 
    outfile : str, optional
        name of png file to store plot 
    xlimits: list of str, 
        xaxis limits  
        if you selected any time options (ymd, mjd, ydoy,mjd), the code assumes
        a format of yyyyMMdd (year,month,day) 
    ylimits: list of floats, optional
        yaxis limits  
    ydoy : bool, optional
        if columns 1 and 2 are year and doy, the x-axis will be plotted in obstimes
        you should select column 1 to plot
    ymd : bool, optional
        if columns 1,2,3 are year, month, date. So meant for plots with daily measurements -
        not subdaily.
    ymdhm : bool, optional
        if columns 1-5 are Y,M,D,H,M then x-axis will be plotted in obstimes
    filename2 : str
        in principle this allows you to make plots from two files with identical formatting
        but I am not sure that it one hundred percent always works
    freq: int, optional
        use column 11 to find (and extract) a single frequency
    utc_offset: int, optional
        offset time axis by this number of hours (for local time)
        this only is used when the mjd option is used  
    yoffset : float
        add or subtract to the y-axis values
    keepzeros: bool, optional
        keep/remove zeros, default is to remove
    sat : str
        satellite number for SNR file plotting 
        for all gps satellites, say gps instead of a number
        similar for glonass, galileo, and beidou
    yoffset2 : float
        add or subtract to the y-axis values in filename2
    scale : float
        multiply all y-axis values in file 1 by this value
    scale2 : float
        multiply all y-axis values in file 2 by this value
    elimits : list of floats
        if SNR file is plotted, elevation angle limits are applied
    azlimits : list of floats
        if SNR file is plotted, azimuth angle limits are applied
    plt: bool
         whether you want the plot to be displayed on the screen.
         png file is always created.

    """
    if xlimits is None:
        xlimits = []

    snrfile = False
    if sat is not None:
        snrfile = True

        if xcol == 'elevation':
            xcol = 1 # python column
        elif xcol == 'time':
            xcol = 3 # python column
        else:
            xcol = int(xcol) - 1

        if (ycol.upper() == 'L1'):
            ycolT = 'L1'
            ycol = 6
        elif (ycol.upper() == 'L2'):
            ycolT = 'L2'
            ycol = 7
        elif (ycol.upper() == 'L5'):
            ycolT = 'L5'
            ycol = 8
        else:
            ycolT = ''
            ycol = int(ycol) - 1
    else:
        # change strings to integers and change to python column
        if (xcol == 'time'):
            print('You cannot chose time for the xcolumn unless you invoke -sat mode. xcol must be an integer. Exit')
            sys.exit()
        elif (xcol == 'elevation'):
            print('You cannot chose elevation for the xcolumn. Unless you invoke -sat mode. xcol must be an integer. Exit')
            sys.exit()
        else:
            xcol = int(xcol) - 1
            ycol = int(ycol) - 1


    if errorcol is None:
        yerrors = False
    else:
        yerrors = True
        errorcol = errorcol - 1

    if ylabel is None:
        ylabel = 'Unknown'

    secondFile = False

    if yoffset is None:
        yoffset = 0
    if yoffset2 is None:
        yoffset2 = 0
    reverse_sign = reverse

    convert_mjd = mjd
    commentsign = '%'

    basename = os.path.basename(filename)
    station = basename[0:4]
    cyear = '20' + basename[9:11]  
    if snrfile:
        xdir = os.environ['REFL_CODE']
        longfile = xdir + '/' + cyear + '/snr/' + station + '/' + basename
        longfile_gz = longfile + '.gz'
        if not os.path.isfile(filename) and os.path.isfile(longfile):
            filename = longfile
            print('using ', longfile)
        elif not os.path.isfile(filename) and os.path.isfile(longfile_gz):
            subprocess.call(['gunzip', longfile_gz])
            filename = longfile
            print('now using ', longfile)

    if (not os.path.isfile(filename)) & os.path.isfile(filename + '.gz'):
        print('I will be nice and gunzip the file for you ...')
        subprocess.call(['gunzip', filename + '.gz'])

    if os.path.isfile(filename):
        tvd = np.loadtxt(filename,comments=commentsign)
        if len(tvd) == 0:
            print('empty input file number 1')
            return
        else:

            if (freq is not None):
                print('restricting frequency',freq)
                ii = (tvd[:,10] == freq)
                if (len(tvd[ii,10]) == 0):
                    print('nothing for that frequency')
                    return
                else:
                    tvd = tvd[ii,:]
            if not keepzeros:
                print('Remove zero values')
                ii = (tvd[:,ycol] != 0)
                tvd = tvd[ii,:]
            if snrfile:
                if sat.lower() == 'gps':
                    print('Show all GPS satellites')
                    print('initial nmber of observations:', len(tvd))
                    ii = (tvd[:,0] < 33)
                    tvd = tvd[ii,:]
                    print('after editing:', len(tvd))
                elif sat.lower() == 'glonass':
                    print('Show all Glonass satellites')
                    print('initial nmber of observations:', len(tvd))
                    ii = (tvd[:,0] > 100) & (tvd[:,0] <= 199)
                    tvd = tvd[ii,:]
                    print('after editing:', len(tvd))
                elif sat.lower() == 'galileo':
                    print('Show all Galileo satellites')
                    print('initial nmber of observations:', len(tvd))
                    ii = (tvd[:,0] > 200) & (tvd[:,0] <= 299)
                    tvd = tvd[ii,:]
                    print('after editing:', len(tvd))
                elif sat.lower() == 'beidou':
                    print('Show all Beidou satellites')
                    print('initial nmber of observations:', len(tvd))
                    ii = (tvd[:,0] > 300) & (tvd[:,0] <= 399)
                    tvd = tvd[ii,:]
                    print('after editing:', len(tvd))
                else:
                    sat = int(sat)
                    print('Only show satellite ', sat)
                    ii = (tvd[:,0] == sat)
                    tvd = tvd[ii,:]
            if len(tvd) == 0:
                print('No data are left. Exiting')
                sys.exit()

            if snrfile:
                # this means it is a SNR file
                if len(elimits) == 2:
                    print('Apply elevation angle limits')
                    e1 = elimits[0]
                    e2 = elimits[1]
                    i= (tvd[:,1] >= e1) & (tvd[:,1] <= e2)
                    tvd=tvd[i,:]
                if len(azlimits) == 2:
                    print('Apply azimuth angle limits')
                    a1 = azlimits[0]
                    a2 = azlimits[1]
                    i= (tvd[:,2] >= a1) & (tvd[:,2] <= a2)
                    tvd=tvd[i,:]
                if len(tvd) == 0:
                    print('no data for this satellite and these editing choices.')
                    print('exiting '); sys.exit()

        tval = []
        yval = []
    else:
        print('input file does not exist')
        sys.exit()


    tvd2=[]
    secondFile = False
    if filename2 is not None:
        print('Warning: filename2 does not allow SNR file options')
        if os.path.isfile(filename2):
            tvd2 = np.loadtxt(filename2,comments=commentsign)
            if len(tvd2) == 0:
                print('empty input for filenumber 2')
                return
            else:
                secondFile = True
                if not keepzeros:
                    print('Remove zero values from second file')
                    ii = (tvd2[:,ycol] != 0)
                    tvd2 = tvd2[ii,:]
                if (sat is not None):
                    print('Only show satellite ', sat, ' for second file')
                    ii = (tvd2[:,0] == sat)
                    tvd2 = tvd2[ii,:]
        else:
            print('second filename does not exist')


    tval,yval = q.trans_time(tvd, ymd, ymdhm, convert_mjd, ydoy,xcol,ycol,utc_offset)

    yval = yval + yoffset

    if secondFile:
        tval2,yval2 = q.trans_time(tvd2, ymd, ymdhm, convert_mjd, ydoy,xcol,ycol,utc_offset)
        yval2 = yval2 + yoffset2
        yval2 = yval2*scale2


    # supercedes previous trans_time ... ??? 
    if ydoy:
        print('Making obstimes for ydoy x-axis')
        tval = g.ydoy2datetime(tvd[:,0], tvd[:,1])
    if ydoy & secondFile:
        print('Making obstimes for second ydoy x-axis')
        tval2 = g.ydoy2datetime(tvd2[:,0], tvd2[:,1])

    fig,ax=myplt.subplots()

    # i.e. using default
    if symbol is None:
        if yerrors:
            ax.errorbar(tval, yval, yerr=tvd[:,errorcol], fmt='.')
        else:
            ax.plot(tval, yval, 'b.')
    else:
        if yerrors:
            ax.errorbar(tval, yval, yerr=tvd[:,errorcol],fmt=symbol)
        else:
            ax.plot(tval, yval, symbol)

    # is second file currently supported???
    if secondFile:
        if symbol is None:
            ax.plot(tval2, yval2, 'r.')
        else:
            # ??
            ax.plot(tval2, yval2, symbol )

    myplt.grid()
    myplt.ylabel(ylabel)

    if xlabel is not None:
        myplt.xlabel(xlabel)
    else:
        if snrfile:
            if (xcol == 1): # python column name
                myplt.xlabel('elevation angle (degrees)')
            if (xcol == 3): # using python column name
                myplt.xlabel('seconds of the day')
            myplt.ylabel(ycolT + ' SNR, dBHz')


    if title is None:
        if snrfile:
            sname = os.path.basename(filename)
            ytitle = sname[0:4] + ' 20' + sname[9:11] + ' ' + sname[4:7]
            ax.set_title('sat ' + str(sat) + '/' + ytitle  )
        else:
            ax.set_title(os.path.basename(filename) )
    else:
        ax.set_title(title)



    if len(ylimits) == 2:
        print('found y-axis limits')
        myplt.ylim((ylimits))
    if len(xlimits) == 2:
        print('found x-axis limits')
        if (convert_mjd | ymd | ydoy | ymdhm):
            if (len(xlimits[0]) != 8):
                print('xlimit format is YYYYMMDD, but you said: ', xlimits[0], ' Exiting'); sys.exit()
            if (len(xlimits[1]) != 8):
                print('xlimit format is YYYYMMDD, but you said: ', xlimits[1], ' Exiting'); sys.exit()
        else:
            # make sure you change from string to numbers
            xlimits[0] = float(xlimits[0])
            xlimits[1] = float(xlimits[1])

        if convert_mjd:
            # need to just get MJD so you can use the local UTC offset
            year1, month1, day1, doy1, mjd1 = g.noaa2me(xlimits[0])
            year2, month2, day2, doy2, mjd2 = g.noaa2me(xlimits[1])

            t1 = Time(mjd1,format='mjd')
            t1_utc = t1.utc # change to UTC
            tvalues1 =  t1_utc.datetime # change to datetime
            t2 = Time(mjd2,format='mjd')
            t2_utc = t2.utc # change to UTC
            tvalues2 =  t2_utc.datetime # change to datetime
            myplt.xlim((tvalues1,tvalues2))
            if utc_offset is not None:
                cc = '{:02d}'.format(abs(utc_offset)) + ':00'
                if utc_offset < 0:
                    myplt.xlabel('UTC-' + cc)
                else:
                    myplt.xlabel('UTC+' + cc)
        else:
            if ymd | ydoy | ymdhm :
                t1 = g.noaatime_to_obstime(xlimits[0])
                t2 = g.noaatime_to_obstime(xlimits[1])
                myplt.xlim((t1,t2))
            else: 
                # this is for when you are plotting something that isn't time
                myplt.xlim((xlimits[0], xlimits[1]))

    if reverse_sign:
        ax.invert_yaxis()
    fig.autofmt_xdate() # obstimes

    q.save_plot(outfile)
    if plt:
        myplt.show()


def main():
    args = parse_arguments()
    run_quickplt(**args)

if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import argparse
import wget
import os
import subprocess
import sys
from shutil import which 


def newchip_gfzrnx(exedir):
    """
    installs the gfzrnx executable and stores in the EXE directory

    Parameters
    ----------
    exedir : str
        location of the executable directory

    """
    savename = 'gfzrnx'
    finalpath = exedir + '/' + savename
    if os.path.exists(finalpath):
        print('The gfzrnx executable already exists')
    else:
        print('Installing gfzrnx')
        localname = 'gfzrnx_osxarm64'
        dfile = 'https://morefunwithgps.com/public_html/' + localname
        wget.download(dfile, savename)
        os.chmod(savename,0o777)
        if os.path.exists(savename):
            subprocess.call(['mv', '-f',savename, exedir])
    return

def newchip_hatanaka(exedir):
    """
    compiles hatanaka code if an existing executable is not there
    stores in EXE directory

    Parameters
    ----------
    exedir : str
        location of the executable directory

    """
    savename = 'CRX2RNX'
    finalpath = exedir + '/' + savename
    if os.path.exists(finalpath):
        print('This Hatanaka executable already exists')
    else:
        print('The Hatanaka CRX2RNX source code will be compiled. This requires gcc.')
        exists = which('gcc')
        if exists is None:
            print('gcc does not exist')
            sys.exit()

        sourcefile = 'crx2rnx.c'; sourceexe =  'crx2rnx.e'
        print('local sourcefile ','docs/' + sourcefile)
        if os.path.exists('docs/' + sourcefile):
            print('Hatanaka source code found locally')
            s = 'docs/' + sourcefile
            subprocess.call(['gcc', s, '-o', sourceexe])
        else:
            print('Hatanaka source code found at github')
            cfile = 'https://raw.githubusercontent.com/kristinemlarson/gnssrefl/master/docs/crx2rnx.c'
            print(cfile)
            wget.download(cfile, sourcefile)
            subprocess.call(['gcc', sourcefile, '-o', sourceexe])
        if os.path.exists(sourceexe):
            print('Hatanaka success - moving to EXE directory')
            subprocess.call(['mv', '-f', sourceexe, finalpath])
            subprocess.call(['rm', '-f', sourcefile])
        else:
            print('no Hatanaka success')
            sys.exit()

    return

def checkexist(exe):
    """
    check to see if an executable exists

    Parameters
    ----------
    exe : str
        executable name to check

    """

    exists = which(exe)
    if exists is None:
        print(exe + ' does not exist on your system. You need to install it.')


def download_chmod_move(url,savename,exedir):
    """
    download an executable, chmod it, and store it locally

    Parameters
    -----------
    url : string
        external location of the executable

    savename : string
        name of the executable

    exedir : string
        name of local executable directory (EXE environment variable)

    """
    f = exedir + '/' + savename
    if os.path.exists(f):
        print('You already have this executable: '+ savename)
    else:
        wget.download(url,savename)
        os.chmod(savename,0o777)
        subprocess.call(['mv', '-f',savename, exedir])
        print('\n Executable stored:', savename)


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("opsys", help="operating system (linux64, macos, or mac-newchip)", type=str)
    args = parser.parse_args().__dict__

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def installexe(opsys: str):
    """
    Command line interface to install non-python executables, specifically
    CRX2RNX and gfzrnx. 

    https://stackoverflow.com/questions/12791997/how-do-you-do-a-simple-chmod-x-from-within-python


    Parameters
    ----------
    opsys : string
        operating system. Allowed values are linux64,  macos, and mac-newchip 
        PC users should use the docker, where these executables 
        come pre-installed

    """

    exedir = os.environ['EXE']
    if not os.path.exists(exedir):
        print('You need to define the EXE environment variable. Exiting')
        sys.exit()
    else:
        print('Your executable environment area: ', exedir)

    checkexist('gzip')
    checkexist('xz')
    checkexist('unzip')
    checkexist('compress')
    checkexist('wget')

    # where the executable files are (currently) stored publicly

    sto = 'https://morefunwithgps.com/public_html/'

    if (opsys == 'linux64'):
        print('Only 64 bit static versions will be provided.')
        print('For 32 bit you will need to check the appropriate websites.')
        savename = 'CRX2RNX'
        url = sto + savename + '.' + opsys + '.e'
        download_chmod_move(url, savename, exedir)

        savename = 'gfzrnx'
        url = sto + 'gfzrnx.' + opsys + '.e'
        download_chmod_move(url, savename, exedir)

        savename = 'teqc'
        if os.path.exists(exedir + '/' + savename):
            print('Executable already exists:', savename)

        else:
            # static executable 64bit
            url = 'https://www.unavco.org/software/data-processing/teqc/development/teqc_CentOSLx86_64s.zip'
            print('Downloading teqc from: ', url)
            try:
                wget.download(url, savename + '.zip')
                subprocess.call(['unzip', savename + '.zip' ])
                subprocess.call(['mv', '-f', savename, exedir])
                subprocess.call(['rm', '-f', savename + '.zip' ])
                print('\n Executable stored:', savename)
            except:
                print('Some kind of kerfuffle trying to install teqc')

    elif (opsys == 'macos'):
        savename = 'CRX2RNX'
        url = sto + savename + '.' + opsys + '.e'
        download_chmod_move(url, savename, exedir)

        savename = 'teqc'
        if os.path.exists(exedir + '/' + savename):
            print('Executable already exists:', savename)
        else:
        # added 2021sep13
            url = 'https://www.unavco.org/software/data-processing/teqc/development/teqc_OSX_i5_gcc4.3d_64.zip'
            print('Downloading teqc from: ', url)
            try:
                wget.download(url, savename + '.zip')
                subprocess.call(['unzip', savename + '.zip' ])
                subprocess.call(['mv', '-f', savename, exedir])
                subprocess.call(['rm', '-f', savename + '.zip' ])
                print('\n Executable stored:', savename)
            except:
                print('Some kind of kerfuffle trying to install teqc')

        savename = 'gfzrnx'
        url = sto + 'gfzrnx.' + opsys + '.e'
        download_chmod_move(url, savename, exedir)
    elif (opsys == 'mac-newchip'):
        print('There is no teqc executable for this architecture, so none will be installed.')
        newchip_hatanaka(exedir)
        newchip_gfzrnx(exedir)

    else:
        print('We do not recognize your operating system input. Exiting.')
        sys.exit()


def main():
    """
    command line code that downloads helper GNSS codes: Hatanaka and gfzrnx

    Parameters
    ----------
    opsys : string
        operating system. Allowed values are linux64,  macos, and mac-newchip
        PC users should use the docker, where these executables
        come pre-installed

    """
    args = parse_arguments()
    installexe(**args)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import argparse
import gnssrefl.gps as g

def main():
    """
    Calculates GPS week information and prints it to the screen

    Parameters
    ----------
    year : integer
        full year
    month : integer
        calendar month
    day : integer
        day of the month

    Returns
    -------
    wk : int
        GPS week
    dayofthewk : int
        day of the week (from 0-6) used by the 
        IGS and others for orbit filenames

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("month", help="month", type=int)
    parser.add_argument("day", help="day", type=int)

    args = parser.parse_args()
    year = args.year
    month = args.month
    day = args.day

    wk,swk = g.kgpsweek(year, month, day, 0,0,0)
    print('WEEK:', wk, ' DAY:', int(swk/86400))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
import argparse
import wget
import sys
import os
import gnssrefl.gps as g


def main():
    """
    Extracts coordinates for stations that were in the UNR database 
    in late 2021. Prints both geodetic and cartesian values, and height
    above sea level.


    Parameters
    ----------
    station : str
        four character station name

    """

    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="4-ch station name (lowercase)", type=str)

    args = parser.parse_args()

    station = args.station
    if len(station) != 4:
        print('illegal station name-must be 4 characters')
        sys.exit()
    alat,alon,ht=g.queryUNR_modern(station.lower())
    coords_found = True
    if (alat+alon+ht) == 0:
        coords_found = False

    if coords_found:
        x,y,z=g.llh2xyz(alat,alon,ht)
        print('XYZ (m):', round(x,4),round(y,4),round(z,4) )
        print('LLH (deg,deg,m):', round(alat,6),round(alon,6),round(ht,3) )

    # attempt to find height above sea level
    foundfile = g.checkEGM()
    if (foundfile) and (coords_found):
        geoidC = g.geoidCorrection(alat,alon)
        sealevel = ht - geoidC
        print('Sea Level (m): ', round(sealevel,3))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
downloads RINEX files
"""
import argparse
import gnssrefl.gps as g
import gnssrefl.karnak_libraries as k
import gnssrefl.highrate as ch
import gnssrefl.rinex2snr as r
import os
import subprocess
import sys
from gnssrefl.utils import str2bool


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("month", help="month (or day of year)", type=int)
    parser.add_argument("day", help="day (zero if you use day of year earlier)", type=int)
    # optional arguments
    parser.add_argument("-rate", default='low', metavar='low', type=str, help="sample rate: low or high")
    parser.add_argument("-archive", default=None, help="archive name", type=str)
    parser.add_argument("-version", default=None, metavar=2, type=int, help="rinex version (2 or 3)")
    parser.add_argument("-strip", default=None, type=str, help="set to True to strip to only SNR observables, gfzrnx used")
    parser.add_argument("-doy_end", default=None, type=int, help="last day of year to be downloaded")
    parser.add_argument("-stream", default=None, type=str, help="set to S get stream-defined Rinex3 filename. Default is R. ")
    parser.add_argument("-samplerate", default=None, type=int, help="Sample rate in seconds. For RINEX3 only.")
    parser.add_argument("-screenstats", default=None, type=str, help="debugging flag for printout. default is False")
    parser.add_argument("-dec", default=None, type=int, help="decimation value (seconds). Only for RINEX 3.")
    parser.add_argument("-save_crx", default=None, type=str, help="Save crx version. Only for RINEX 3.")
    parser.add_argument("-delete_hourly", default=None, type=str, help="Delete hourly files from NGS archive?. Default is yes")

    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['strip','screenstats','save_crx','delete_hourly']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def download_rinex(station: str, year: int, month: int, day: int, rate: str = 'low', archive: str = 'all',
                   version: int = 2, strip: bool = False, doy_end: int = None, stream: str = 'R', samplerate: int = 30, 
                   screenstats : bool = False, dec: int = 1, save_crx: bool = False, delete_hourly: bool = True ):
    """
    Command line interface for downloading RINEX files from global archives.
    Required inputs are station, year, month, and day. If you want to use day of year,
    call it as station, year, doy, 0.

    decimate does not seem to do anything, at least not for RINEX 2.11 files

    bkg option is changed.  now must specify bkg-igs or bkg-euref

    Examples
    --------

    download_rinex mfle 2015 1 1 
        downloads January 1, 2015

    download_rinex mfle 2015 52 0
        Using day of year instead of month/day:

    download_rinex p101 2015 52 0 -archive sopac
        checks only sopac archive

    Parameters
    ----------
    station : str
        4 or 9 character ID of the station.

    year : int
        full Year

    month : int
        month

    day : int
        day of month

    rate : str, optional
        sample rate. value options:

            low (default) : standard rate data

            high : high rate data

    archive : str, optional
        Select which archive to get the files from.
        Default is redirected to all, as defined below. Value options:

            cddis : (NASA)

            bev : (Austria Federal Office of Metrology and Surveying)

            bkg-igs : igs folder of BKG (German Agency for Cartography and Geodesy)

            bkg-euref : Euref folder of BKG (German Agency for Cartography and Geodesy)

            bfg : (German Agency for water research, only Rinex 3)

            ga : (Geoscience Australia)

            gfz : (GFZ)

            jp : (Japan)

            jeff : Jeff Freymueller

            nrcan : (Natural Resources Canada)

            ngs : (National Geodetic Survey)

            nz : (GNS, New Zealand)

            sonel : (?)

            sopac : (Scripps Orbit and Permanent Array Center)

            special : (reflectometry Rinex 2.11 files maintained by unavco)

            unavco : now earthscope

            ngs-hourly: NGS, hourly files will be merged if they exist

            all : (searches unavco, sopac, and sonel in that order)

    version : int, optional
        Version of Rinex file. Default is 2.
        Value options 2 or 3 

    strip : bool, optional
        Whether to strip only SNR observables.  Uses teqc or gfzrnx.
        Default is False.

    doy_end : int, optional
        End day of year to be downloaded. 
        Default is None. (meaning only a single day using the doy parameter)

    stream : str, optional
        Receiver or stream file, for RINEX3 only
        Default is 'R' but you can set to 'S' to get streamed version

    samplerate : int, optional
        Sample rate in seconds for RINEX3 only.
        Default is 30.

    screenstats : bool, optional
        provides screen output helpful for debugging
        Default is False

    dec : int, optional
        some highrate file downloads allow decimation. Default is 1 sec, i.e. no decimation

    save_crx : bool, option
        saves crx version for Rinex3 downloads. Otherwise they are deleted.

    """

#   make sure environment variables exist.  set to current directory if not

    g.check_environ_variables()
    debug = screenstats

    if 'bkg' in archive:
        if (archive == 'bkg'):
            print('You have not specified which BKG archive you want.')
            print('You must select bkg-igs or bkg-euref.')
            sys.exit()
        if 'euref' in archive:
            bkg = 'EUREF'
        else:
            bkg = 'IGS'
        # change archive name back to original
        archive = 'bkg'

    if len(str(year)) != 4:
        print('Year must have four characters: ', year)
        sys.exit()

    if (samplerate == 1):
        # even if you forget to set it to high ...
        rate = 'high'

    month, day, doy, cyyyy, cyy, cdoy = g.ymd2ch(year,month,day)

    # allowed archives, rinex 2.11
    # not really sure bev, gfz, bkg work ???
    archive_list = ['bkg','bfg','bev','cddis', 'ga', 'gfz', 'jeff', 'ngs', 
            'nrcan', 'nz','sonel','sopac','special', 'unavco', 'all','unavco2','ngs-hourly']

    # removed the all archive
    # removed cddis because it is too slow
    archive_list_high = ['bkg','unavco', 'nrcan', 'ga']  # though it is confusing because some are rinex 2.11 and others 3

    # archive list for rinex3 lowrate files
    archive_list_rinex3 = ['unavco', 'bkg','cddis', 'ga', 'bev', 'ign', 'epn', 'bfg','sonel','all','unavco2','nrcan','gfz']

    if doy_end is None:
        doy_end = doy

    NS = len(station)

    if NS == 9:
        version = 3 # even if you don't choose version 3 .... 
    
    # this is for version 2
    # not going to allow GSI for now
    if (version == 2) and (rate == 'low'):
        if NS != 4:
            if not archive == 'jp':
                print('exiting: RINEX 2.11 station names must have 4 characters, lowercase please')
                sys.exit()

        if archive is None:
            archive = 'all'
        else:
            archive = archive.lower()
            if archive not in archive_list:
                print('You picked an archive that does not exist. To be nice, ')
                print('I am going to check the main ones (unavco,sopac,sonel) for you')
                print('For future reference: I allow these archives:')
                print(archive_list)
                archive = 'all'

    #print('archive selected: ' , archive)
    # default archive wil be CDDIS for version 3
    if version == 3:
        if NS != 9:
            print('exiting: RINEX 3+ station names must have 9 characters')
            sys.exit()
        if archive is None:
            archive = 'cddis'

        if stream not in ['R', 'S']:
            stream = 'R'

        if archive == 'unavco':
            # override the sample rate here
            samplerate = 15

        if archive == 'unavco2':
            # override the sample rate here
            samplerate = 15

        if archive not in archive_list_rinex3:
            print('You picked an archive that is not supported by my code. Exiting')
            print(archive_list_rinex3)
            sys.exit()

    if (rate == 'high') and (version == 2):
        if archive is None:
            archive = 'unavco'

        if archive not in archive_list_high:
            print('You picked an archive that is not supported by my code. Exiting')
            sys.exit()

    if (rate == 'high') and (version == 3):
        if (archive == 'bkg') or (archive == 'ga'):
            print('Highrate RINEX 3 is supported - but it is very slow. Pick up a cup of coffee.')
        else:
            print('I do not support RINEX 3 high rate downloads from your selected archive.')
            sys.exit()

    for d in range(doy, doy_end+1):
        # RINEX Version 3
        if version == 3:
            print('Request ', station, year, d, archive, samplerate, stream)
            if rate == 'high':
                if archive == 'cddis':
                    bad_day = g.cddis_restriction(year, doy,'cddis')
                    print('seek highrate data at CDDIS')
                    if bad_day:
                        rnx_filename, foundit = ch.cddis_highrate_tar(station, year, d, 0, stream, 1)
                    else:
                        rnx_filename, foundit = ch.cddis_highrate(station, year, d, 0, stream, 1)
                if archive == 'bkg':                               
                    print('seek highrate data at BKG')
                    bad_day = g.cddis_restriction(year, doy,'bkg')
                    if bad_day:
                        rnx_filename,foundit = ch.bkg_highrate_tar(station, year, d, 0,stream,dec,bkg)
                    else:
                        rnx_filename,foundit = ch.bkg_highrate(station, year, d, 0,stream,dec,bkg)
                if archive == 'ga':
                    if debug:
                        print('seek highrate data at GA')
                    deleteOld = True
                    r2, foundit = g.ga_highrate(station,year,d,dec,deleteOld)
            else:
                if archive == 'all':
                    file_name, foundit = k.universal_all(station, year, d, samplerate, stream,debug)
                    if not foundit:
                        file_name, foundit = k.universal_all(station, year, d, samplerate, k.swapRS(stream),debug)
                else:
                    file_name, foundit = k.universal(station, year, d, archive, samplerate, stream,debug)
                    if not foundit:
                        file_name, foundit = k.universal(station, year, d, archive, samplerate, k.swapRS(stream),debug)
                if foundit: 
                    print('\n SUCCESS 1: ', file_name)
                    deletecrx = not save_crx
                    translated, new_file_name = r.go_from_crxgz_to_rnx(file_name,deletecrx)
                    if translated:
                        # i do not think this was doing what we thought it was doing ....
                        #subprocess.call(['rm', '-f', new_file_name.replace('rnx', 'crx')])  # delete crx file
                        print('\n SUCCESS 2: ', new_file_name)
                else:
                    print('Unsuccessful RINEX 3 retrieval')

        else:  # RINEX VERSION 2
            # using new karnak code
            rinexfile, rinexfiled = g.rinex_name(station, year, d, 0)
            # gets hourly downloads, so not highrate in the regular sense
            if (archive == 'ngs-hourly'):
                print('try to find hourly files at the NGS')
                g.big_Disk_work_hard(station,year,d,0,delete_hourly)
                sys.exit()

            if rate == 'high':
                rinexfile, foundit = k.rinex2_highrate(station, year, d, archive, strip)
            else:
                if archive == 'all':
                    foundit = False
                    if debug:
                        print('cycle thru unavco,sopac,sonel archives')
                    for archiveinput in ['unavco', 'sopac', 'sonel']:
                        if not foundit:
                            file_name, foundit = k.universal_rinex2(station, year, d, archiveinput,debug)
                else:
                    file_name, foundit = k.universal_rinex2(station, year, d, archive, debug)
                if foundit:  # uncompress and make o files ...
                    rinexfile, foundit = k.make_rinex2_ofiles(file_name)  # translate

            if foundit:
                print('SUCCESS:', rinexfile, ' was found')
            else:
                print('FAILURE:', rinexfile, ' was not found')

            if os.path.isfile(rinexfile) and (strip is True):
                k.strip_rinexfile(rinexfile)


def main():
    args = parse_arguments()
    download_rinex(**args)


if __name__ == "__main__":
    main()



from scipy.interpolate import interp1d
import math
import numpy as np
import os
import subprocess
import sys
import time
import wget

import gnssrefl.gps as g

import simplekml


# https://developers.google.com/kml/documentation/kml_tut#ground-overlays


def makeFresnelEllipse(A,B,center,azim):
    """
    make an Fresnel zone given size, center, and orientation

    Parameters
    ----------
    A : float
        semi-major axis of ellipse in meters
    B : float
        semi-minor axis of ellipse in meters
    center : float 
        center of the ellipse, provided as distance along the satellite azimuth direction
    azimuth : float
        azimuth angle of ellipse in degrees. 
        this will be clockwise positive as defined from north

    Returns
    --------
    x : numpy array of floats 
        x value of cartesian coordinates of ellipse
    y : numpy array of floats
        y value of cartesian coordinates of ellipse
    xcenter : float
        x value for center of ellipse in 2-d cartesian
    ycenter : float
        y value for center of ellipse in 2-d cartesian

    """

    # this is backwards but it works and i have stopped caring
    width =  A
    height = B
    # fro my matlab code
    angle = 360-azim  + 90

    allAngles = np.deg2rad(np.arange(0.0, 375.0, 15.0))
    # get the x and y coordinates for an ellipse centered at 0,0
    x =  width * np.cos(allAngles)
    y =  height * np.sin(allAngles)

    # rotation angle for the ellipse
    rtheta = np.radians(angle)
    # rotation array
    R = np.array([
        [np.cos(rtheta), -np.sin(rtheta)],
        [np.sin(rtheta),  np.cos(rtheta)], ])
    

#   rotate x and y into this new system
    x, y = np.dot(R, np.array([x, y]))
#   figure out center of the ellipse
    xcenter = center*np.cos(rtheta)
    ycenter = center*np.sin(rtheta)
    # print('center of the ellipse', xcenter, ycenter)
# finally, new coordinates for x and y
    x += xcenter
    y += ycenter
    return x, y, xcenter, ycenter


def FresnelZone(f,e,h):
    """
    based on GPS Tool Box Roesler and Larson (2018).
    Original source is Felipe Nievinski as published in the appendix of 
    Larson and Nievinski 2013
    this code assumes a horizontal, untilted reflecting surface    

    Parameters
    ----------
    f : int
        frequency (1,2, or 5)
    e : float
        elevation angle (deg)
    h : float
        reflector height (m)

    Returns
    -------
    firstF: list of floats 
        [a, b, R ] in meters where:
        a : is the semi-major axis, aligned with the satellite azimuth 
        b : is the semi-minor axis
        R : locates the center of the ellispe on the satellite azimuth direction (theta)

    """

# SOME GPSCONSTANTS	
    CLIGHT = 299792458;  # speed of light, m/sec
    FREQ = [0, 1575.42e6, 1227.6e6, 0, 0, 1176.45e6];   # GPS frequencies, Hz
    CYCLE = CLIGHT/FREQ[f]; #  wavelength per cycle (m/cycle)
    RAD2M = 0.5*CYCLE/np.pi; # % (m)
    erad = e*np.pi/180;

# check for legal frequency later

# delta = locus of points corresponding to a fixed delay;
# typically the first Fresnel zone is is the 
# "zone for which the differential phase change across
# the surface is constrained to lambda/2" (i.e. 1/2 the wavelength)
    delta = CYCLE/2; # 	% [meters]


# semi-major and semi-minor dimension
# from the appendix of Larson and Nievinski, 2013
    sin_elev = math.sin(erad);
    d = delta; 
    B = math.sqrt( (2*d*h / sin_elev) + (d/sin_elev)*(d/sin_elev) ) ; # % [meters]
    A = B / sin_elev ; #% [meters]


# determine distance to ellipse center 
    center = (h + delta/sin_elev)/ math.tan(erad)  #  	% [meters]
#    print('center distance is', center)

    return A, B, center

def makeEllipse_latlon(freq,el,h,azim,latd,lngd):
    """
    for given fresnel zone, produces coordinates of an ellipse

    Parameters
    ----------
    freq : int
        frequency
    el : float
        elevation angle in degrees
    h : float
        reflector height in meters
    azim : float
        azimuth in degrees
    latd : float
        latitude in degrees
    lngd : float
        longitude in degrees

    Returns
    -------
    lngdnew : float
        new longitudes in degrees

    latdnew : float
        new latitudes in degrees

    """
    A,B,center = FresnelZone(freq,el,h) 
    # print(A,B,center)
    x,y,xc,yc = makeFresnelEllipse(A,B,center,azim)
    d=np.sqrt(x*x+y*y); # this is in meters  
    d=d/1000; # convert d from meters to km
    # average Radius of Earth, in km
    R=6378.14; 
    theta=np.arctan2(x,y)
# map coordinates need a reference, so use the station coordinates, in degrees and radians
    lat = latd*np.pi/180
    lng = lngd*np.pi/180
    sinlat = math.sin(lat)
    coslat = math.cos(lat)

# this will be in radians
    latnew= np.arcsin(sinlat*np.cos(d/R) + coslat*np.sin(d/R)*np.cos(theta) );

    arg1 =  np.sin(theta)*np.sin(d/R)*coslat
    arg2 =  np.cos(d/R) - sinlat*np.sin(latnew)
# new lngnew will be degrees
    lngnew = lngd + 180.0/np.pi  * np.arctan2(arg1, arg2)
# put latitude back into degrees
    latnew=latnew*180./np.pi 

    return lngnew, latnew

def set_final_azlist(a1ang,a2ang,azlist):
    """
    edits initial azlist to restrict to given azimuths

    Parameters
    ----------
    a1ang : float
        minimum azimuth (degrees)
    a2ang : float
        maximum azimuth (degrees)
    azlist : list of floats 
        list of tracks, [azimuth angle, satNumber, elevation angle ]

    Returns
    -------
    azlist : list of floats

    """
# we convert the azimuth tracks to -180 to 180, sort of
    if (a1ang < 0):
        kk = (azlist[:,0] > 0) & (azlist[:,0] < a2ang)
        kk2 = (azlist[:,0] > (360+a1ang)) & (azlist[:,0] < 360)
        azlist1 = azlist[kk,:]
        azlist2 = azlist[kk2,:]
        azlist = np.vstack((azlist1, azlist2))
    # these might overlap if you put in silly numbers but i don't think it will crash
    else:
        kk = (azlist[:,0] > a1ang) & (azlist[:,0] < a2ang)
        azlist = azlist[kk,:]

    return azlist


def set_azlist_multi_regions(sectors,azlist):
    """
    edits initial azlist to restrict to given azimuth sectors.
    assumes that illegal list of sectors have been checked (i.e.
    no negative azimuths, they should be pairs, and increasing)

    Parameters
    ----------
    sectors: list of floats
        min and max azimuth (degrees). Must be in pairs, no negative numbers
    azlist : list of floats
        list of tracks, [azimuth angle, satNumber, elevation angle ]

    Returns
    -------
    azlist2 : list of floats
         same format as before, but with azimuths removed outside the restricted zones

    """
    npairs = int(len(sectors)/2)
    azlist2 = np.empty(shape=[0, 3])
    for n in range(0,npairs):
        a1 = sectors[n*2] # min azim of sector
        a2 = sectors[n*2+1] # max zzim of sector
        kk = (azlist[:,0] > a1) & (azlist[:,0] < a2)
        azlistsec = azlist[kk,:]
        azlist2 = np.vstack((azlist2, azlistsec))

    return azlist2

def calcAzEl_new(prn, newf,recv,u,East,North):
    """
    function to gather azel for all low elevation angle data
    this is used in the reflection zone mapping tool

    Parameters
    ----------
    prn : int
        satellite number
    newf : 3 vector of floats
        cartesian coordinates of the satellite (meters)
    recv : 3 vector of floats
        receiver coordinates (meters)
    u : 3 vector
        cartesian unit vector for up
    East : 3 vector
        cartesian unit vector for east direction
    North : 3 vector
        cartesian unit vector for north direction

    Returns
    -------
    tv : numpy array of floats
        list of satellite tracks
        [prn number, elevation angle, azimuth angle]
    """
    tv = np.empty(shape=[0, 3])
    # number of values
    NV = len(newf)
    #for t in range(0,480):
    for t in range(0,NV):
        satv= [newf[t,2], newf[t,3],newf[t,4]]
        etime = newf[t,1]
        r=np.subtract(satv,recv) # satellite minus receiver vector
        eleA = g.elev_angle(u, r)*180/np.pi
        #Check if the elevation angle is within the allowed range
        if ( (eleA >= 0) & (eleA <= 61)):
            azimA = g.azimuth_angle(r, East, North)
#            print(etime, eleA, azimA)
            newl = [prn, eleA, azimA]
            tv = np.append(tv, [newl],axis=0)
    nr,nc=tv.shape
    return tv


def rising_setting_new(recv,el_range,obsfile):
    """
    Calculates potential rising and setting arcs

    Parameters
    ----------
    recv : list of floats
         Cartesian coordinates of station in meters
    el_range : list of floats
         elevation angles in degrees
    obsfile : str
         orbit filename

    Returns
    -------
    azlist : list of floats
        azimuth angle (deg), PRN, elevation angle (Deg)

    """
    # will put azimuth and satellite number
    azlist = np.empty(shape=[0, 3])
    # these are in degrees and meters
    lat, lon, nelev = g.xyz2llhd(recv)
    # calculate unit vectors
    u, East,North = g.up(np.pi*lat/180,np.pi*lon/180)
    # load the cartesian satellite positions
    f = np.genfromtxt(obsfile,comments='%')
    r,c = f.shape
    # print('Number of rows:', r, ' Number of columns:',c)
#   reassign the columns to make it less confusing
    sat = f[:,0]; t = f[:,1]; x = f[:,2]; y = f[:,3]; z =  f[:,4]
#   examine all 32 satellites
#   should this be more for Beidou?
    for prn in range(1,33):
        newf = f[ (f[:,0] == prn) ]
        tvsave=calcAzEl_new(prn, newf,recv,u,East,North)
        nr,nc=tvsave.shape
        for e in el_range:
            #print('elevation angle: ', e)
            el = tvsave[:,1] - e
            for j in range(0,nr-1):
                az = round(tvsave[j,2],2)
                newl = [az, prn, e]
                if ( (el[j] > 0) & (el[j+1] < 0) ) :
                    azlist = np.append(azlist, [newl], axis=0)
                if ( (el[j] < 0) & (el[j+1] > 0) ) :
                    azlist = np.append(azlist, [newl], axis=0)

    return azlist

def write_coords(lng, lat):
    """
    Parameters
    ----------
    lng : list of floats
        longitudes in degrees
    lat : list of floats 
        latitudes in degrees

    Returns
    -------
    points : list of pairs  of long/lat
        for google maps

    """
    points = []
    data = [lng, lat]
    for i in range(len(lng)):
        coord = (data[0][i], data[1][i])
        points.append(coord)
        i+=1

    return points

# Function makes the fresnel zones to KML file
def make_FZ_kml(station, filename,freq, el_list, h, lat,lng,azlist):
    """
    makes fresnel zones for given azimuth and elevation angle 
    lists.  

    Parameters
    ----------
    station: str
        four character station name
    filename : str
        output filename (the kml extension should already be there)
    freq : int
        frequency (1,2, or 5)
    el_list : list of floatss
        elevation angles
    h : float
        reflector height in meters
    lat : float
        latitude in deg
    lng : float 
        longitude in degrees
    azlist : list of floats
        azimuths

    """
    # starting azimuth
    nr,nc=azlist.shape
    n=0
    kml = simplekml.Kml()
    # this loop goes through all the Fresnel zone azimuths in azlist
    while (n < nr):
        azim = azlist[n,0]; el = azlist[n,2]
        prn = int(azlist[n,1])
        k = el_list.index(el) ; # color index
        lng_el, lat_el = makeEllipse_latlon(freq,el,h,azim, lat,lng)
        points = write_coords(lng_el, lat_el)
        #pname = 'prn {0} elev'.format(prn)
        pname = 'PRN:' + str(prn) + ' elev:' + str(int(el))
        #pname = 'ElevAngle {0}'.format(int(el), prn)
        ls = kml.newpolygon(name=pname, altitudemode='relativeToGround') # creating new polygon for each azimuth zone in azlist
        ls.outerboundaryis = points
        # wont work ...
        # print(points)
        #ls.labelstyle.scale = 5
        if el ==el_list[0]:
            ls.style.linestyle.color = simplekml.Color.yellow
            ls.style.linestyle.width = 3
            ls.style.polystyle.color = simplekml.Color.changealphaint(50, simplekml.Color.yellow)
        elif el==el_list[1]:
            ls.style.linestyle.color = simplekml.Color.blue
            ls.style.linestyle.width = 3
            ls.style.polystyle.color = simplekml.Color.changealphaint(50, simplekml.Color.blue)
        elif el==el_list[2]:
            ls.style.linestyle.color = simplekml.Color.red
            ls.style.linestyle.width = 3
            ls.style.polystyle.color = simplekml.Color.changealphaint(50, simplekml.Color.red)
        elif el==el_list[3]:
            ls.style.linestyle.color = simplekml.Color.green
            ls.style.linestyle.width = 3
            ls.style.polystyle.color = simplekml.Color.changealphaint(50, simplekml.Color.green)
        elif el==el_list[4]:
            ls.style.linestyle.color = simplekml.Color.cyan
            ls.style.linestyle.width = 3
            ls.style.polystyle.color = simplekml.Color.changealphaint(50, simplekml.Color.cyan)
        else:
            ls.style.polystyle.color = simplekml.Color.white
            ls.style.linestyle.width = 5
        n+=1

    #print('put end of file on the geoJSON')
    azim = azlist[nr-1,0]
    # try this instead of hardwiring 5!
    el=el_list[0]
    #print('elevation angle for last one')
    #print(el_list[0])
    lng_el, lat_el = makeEllipse_latlon(freq,el,h,azim, lat,lng)
    # i do not think this is needed so removed it

    if False:
        points = write_coords(lng_el, lat_el)

    #print(points)
    if False:
        ls = kml.newpolygon(name='A Polygon {0}'.format(n+1))
        ls.outerboundaryis = points
        ls.style.linestyle.color = simplekml.Color.yellow
        ls.style.linestyle.width = 3
        ls.style.polystyle.color = simplekml.Color.changealphaint(50, simplekml.Color.yellow)

    # try adding a point at the station
    pnt = kml.newpoint(name=station)
    #pnt.labelstyle.scale = 3
    pnt.coords = [(lng, lat)]
    pnt.style.iconstyle.icon.href = 'http://maps.google.com/mapfiles/kml/shapes/placemark_circle.png'

    # save to a file
    kml.save(filename)
    return True

def set_system(system):
    """
    finds the file needed to compute orbits for reflection zones

    Parameters
    ----------
    system : str
        gps,glonass,beidou, or galileo
    it : int
        simple pointer from former code. 1 is GPS, 2 is Glonass etc

    Returns
    -------
    orbfile : str
        orbit filename with Cartesian coordinates for one day 

    """
    it = 1
    xdir = os.environ['REFL_CODE'] 
    if not os.path.exists(xdir):
        print('REFL_CODE environment variable must be set. Exiting.')
        sys.exit()
    xdir = os.environ['REFL_CODE'] + '/Files/'
    if not os.path.exists(xdir):
        subprocess.call(['mkdir', xdir])

    if (system is None) or (system == 'gps'):
        system = 'gps'
        orbfile = xdir + 'GPSorbits_21sep17.txt'
        it = 1
    elif (system == 'galileo'):
        orbfile = xdir + 'GALILEOorbits_21sep17.txt'
        it = 3
    elif (system == 'glonass'):
        orbfile = xdir + 'GLONASSorbits_21sep17.txt'
        it = 2
    elif (system == 'beidou'):
        orbfile = xdir + 'BEIDOUorbits_21sep17.txt'
        it = 4
    else:
       print('Using GPS')
       system = 'gps'
       it = 1
       orbfile = xdir + 'GPSorbits_21sep17.txt'

    return orbfile, it 

def save_reflzone_orbits():
    """
    check that orbit files exist for reflection zone code.
    downloads to $REFL_CODE$/Files directory if needed

    Returns
    -------
    foundfiles : bool
        whether needed files were found
    """
    xdir = os.environ['REFL_CODE'] 

    if not os.path.exists(xdir):
        print('The REFL_CODE environment variable must be set. Exiting.')
        sys.exit()

    xdir = os.environ['REFL_CODE'] + '/Files/'

    if not os.path.exists(xdir):
        subprocess.call(['mkdir', xdir])

    githubdir = 'https://raw.githubusercontent.com/kristinemlarson/gnssrefl/master/docs/_static/'

    for otypes in ['GPS','GLONASS','GALILEO','BEIDOU']:
        orbfile = otypes + 'orbits_21sep17.txt'
        #print( githubdir+orbfile)
        if not os.path.exists(xdir + orbfile):
            print('download from github and put in local Files directory: ', otypes)
            wget.download(githubdir+orbfile, xdir + orbfile)
        #else:
        #    print('file already exists', otypes)

    found = 0 
    for otypes in ['GPS','GLONASS','GALILEO','BEIDOU']:
        orbfile = otypes + 'orbits_21sep17.txt'
        if os.path.exists(xdir + orbfile):
            found = found + 1

    foundfiles = False
    if found == 4:
        foundfiles = True

    return foundfiles 

def nyquist_simple(t,elev,azims, emin,emax,azriseset,reqsamplerate):
    """
    given numpy array of elevation angles (elev)
    and limtis (emin,emax) and azriseset, reqsamplerate

    Parameters
    ----------
    t : ??
        cannot remember
    elev : numpy array of floats
        elevation angle of rising/setting arcs
    azims : numpy array of floats
        azimuths of rising/setting arcs
    emin : float
        minimum elevation angle, degrees
    emax : float
        maximum elevation angle, degrees
    azriseset : ??
        cannot remember
    reqsamplerate : float
        requested receiver sampling rate

    """
    # sample rate used for primary orbit calculation.  will scale by requested sample rate
    samplerate = 30
    # t is every five minutes
    ie= interp1d(t, elev, kind='quadratic',bounds_error=False,fill_value='extrapolate')
    ia= interp1d(t, azims, kind='quadratic',bounds_error=False,fill_value='extrapolate')
    tnew = np.arange(t[0], t[-1], samplerate)
    # interporlate el and az
    enew = ie(tnew)
    anew = ia(tnew)
    # window it
    i = (np.abs(anew-azriseset) < 20); enew = enew[i]
    i = (enew >= emin) & (enew <= emax); enew = enew[i]
    # scale factor for L1
    cf = 0.1902936/2
    N = len(enew)
    if (N > 2):
        maxe = max(enew); mine = min(enew)
    # highest observation has to be within 0.5 degrees of requested
        if ( maxe < (emax-0.5)):
        #print(azriseset, 'max ele ', round(maxe,2), ' Not a significant arc')
            nyq = np.nan
        else:
            diffT = (np.sin( maxe*np.pi/180) - np.sin(mine*np.pi/180) ) /cf
    # we do 30 sec and then multiple by ratio of requested sample rate
            nyq = round(N/(2*diffT),2)*(samplerate/reqsamplerate)
    else:
        nyq = np.nan

    return nyq


def calcAzEl_newish(prn, newf,recv,u,East,North):
    """
    should be consolidated with the other function.but who has the time!

    Parameters
    ----------
    prn : int
        satellite number ?

    newf : 

    u : numpy array
        up unit vector
    East: 

    North : 

    Returns
    -------
    tv : numpy array

    """
    tv = np.empty(shape=[0, 4])
    # number of values
    NV = len(newf)
    for t in range(0,NV):
        satv= [newf[t,2], newf[t,3],newf[t,4]]
        etime = newf[t,1]
        r=np.subtract(satv,recv) # satellite minus receiver vector
        eleA = g.elev_angle(u, r)*180/np.pi
        if ( (eleA >= 0) & (eleA <= 20)):
            azimA = g.azimuth_angle(r, East, North)
#            print(etime, eleA, azimA)
            newl = [prn, eleA, azimA, etime]
            tv = np.append(tv, [newl],axis=0)
    nr,nc=tv.shape
    return tv




# -*- coding: utf-8 -*-
import argparse
import datetime
import numpy as np
import os
import sys
import subprocess
import time

import gnssrefl.gps as g
import gnssrefl.nmea2snr as nmea
import gnssrefl.gnssir_v2 as guts2

from gnssrefl.utils import validate_input_datatypes, str2bool

import multiprocessing
from functools import partial


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("doy", help="start day of year", type=int)

    parser.add_argument("-snr", default=66, help="snr file type, 99: 5-30 deg.; 66: < 30 deg.; 88: all data; 50: < 10 deg", type=int)
    parser.add_argument("-year_end", default=None, help="end year", type=int)
    parser.add_argument("-doy_end", default=None, help="end day of year", type=int)
    parser.add_argument("-overwrite", default=None, help="boolean", type=str)
    parser.add_argument("-dec", default=None, help="decimation, seconds", type=int)
    parser.add_argument("-lat", default=None, help="latitude, degrees", type=float)
    parser.add_argument("-lon", default=None, help="longitude, degrees", type=float)
    parser.add_argument("-height", default=None, help="ellipsoid height, m", type=float)
    parser.add_argument("-risky", default=None, help="boolean for whether low quality orbits are used instead of precise sp3", type=str)
    parser.add_argument("-gzip", default=None, help="gzip SNR file after creation. Default is true.", type=str)
    parser.add_argument("-par", default=None, help="parallel processing, up to 10", type=int)
    parser.add_argument("-orb", default=None, help="request specific orbit source", type=str)
    parser.add_argument("-hour", default=None, help="request hour for ultra orbit", type=int)

    g.print_version_to_screen()

    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['risky', 'gzip', 'overwrite']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}

def nmea2snr( station: str, year: int, doy: int, snr: int = 66, year_end: int=None, doy_end: int=None, 
             overwrite : bool=False, dec : int=1, lat : float = None, lon : float=None, 
             height : float = None, risky : bool=False, gzip : bool = True, par:int = None, orb:str = None, hour:int =0):
    """
    This code creates SNR files from NMEA files.  

    The NMEA files should be stored in $REFL_CODE/nmea/ssss/2023 for station ssss and year 2023
    or $REFL_CODE/nmea/SSSS/2023 for station SSSS. The NMEA files should be named 
    SSSS1520.23.A or ssss1520.23.A, where the day of year is 152 and year is 2023 in this example.

    The SNR files created are stored with upper case if given upper case, lower case if given lower case.
    Currently I have left the last character in the file name as it was given to me - capital A.
    If this should be lower case for people that use lowercase station names, please let me know.
    As far as I can tell, the necessary fields in the NMEA files are GPGGA and GPGSV.

    Originally this code used interpolations of the az and el NMEA fields. I have decided this 
    is DANGEROUS. If you really want to use those low-quality measurements, you have to say -risky T

    The default usage is to use multi-GNSS orbits from GFZ.  To compute az-el, you need to 
    provide a priori station coordinates. You can submit those on the 
    command line or it will read them from the $REFL_CODE/input/ssss.json file 
    (for station ssss or SSSS) if it exists.

    As of 2023 September 14, the SNR files are defined in GPS time, which is how the file is defined.
    Prior to version 1.7.0, if you used the sp3 option, the SNR files were written in UTC. This led to 
    the orbits being propagated to the wrong time and thus az-el values are biased. The impact on RH 
    is not necessarily large - but you should be aware. The best thing to do is remake your SNR files.  

    As for March 16, 2024, this code has been changed to use gnssrefl standards for inputs and outputs.
    The code, in principle, now looks for final, rapid, and ultra rapid orbits from GFZ, in that order.

    Parameters
    ----------
    station : str
        4 ch name of station
    year : int
        full year
    doy : int
        day of year
    snr : int, optional
        snr file type (default is 66); 99: 5-30 deg.; 66: < 30 deg.; 88: all data; 50: < 10 deg
    year_end : int, optional
        final year
    doy_end : int, optional
        final day of year
    overwrite : bool, optional
        whether make a new SNR file even if one already exists
    dec : int, optional
        decimation in seconds
    lat: float, optional
        latitude, deg, 
    lon: float, optional
        longitude, deg
    height: float, optional
        ellipsoidal height, m
    risky : bool, optional
        confirm you want to use low quality orbits (default is False)
    gzip : bool, optional
        compress SNR files after creation.  Default is true
    par : int
        number of parallel processes. default is none (i.e. 1)
    orb : str
        specific orbit source (gnss, rapid, ultra, wum2). default is rapid
    hour : int
        specific hour of ultrarapid orbit. default is zero

    Examples
    --------
    nmea2snr wesl 2023 8 -dec 5
         makes SNR file with decimation of 5 seconds with good orbits

    nmea2snr wesl 2023 8 
         makes SNR file with original sampling rate  and good orbits

    nmea2snr xyz2 2023 8 -lat 40.2342 -lon -120.32424 -height 12
         makes SNR file with user provided station coordinates and good orbits

    """
    # queue which handles any exceptions any of the processes encounter
    manager = multiprocessing.Manager()
    error_queue = manager.Queue()


    g.check_environ_variables()

    NS = len(station)
    if (NS != 4):
        print('Illegal input - Station name must have 4 characters. Exiting.')
        sys.exit()

    # THIS CODE DOES NOT USE OUR ACCEPTED PROTOCOLS for argparse
    if len(str(year)) != 4:
        print('Year must be four characters long. Exiting.', year)
        sys.exit()    

    # 
    gfz_date = 2021 + 137/365.25
    # assume people are sensible until proven otherwise

    recv = [0,0,0]
    llh = [lat,lon,height]    

    if risky:
        print('You insist on using low quality az-el NMEA values and have set the risky option to True.')
        sp3 = False
    else:
        sp3 = True
        # try to get a priori coordinates, Cartesian
        recv, foundcoords = nmea.nmea_apriori_coords(station,llh,sp3)
        if not foundcoords:
            print('The default in this code is to use precise orbits to calculate az/el values.')
            print('We need to know apriori coordinates for your site. Please input lat/lon/ellipsoidal height ')
            print('on the command line or set those values using gnssir_input.')
            sys.exit()


    if doy_end is None:
        doy_end = doy
    if year_end is None:
        year_end = year



    MJD1 = int(g.ydoy2mjd(year,doy))
    MJD2 = int(g.ydoy2mjd(year_end,doy_end))

    #def run_nmea2snr(station, year, doy, isnr, overwrite, dec, llh, sp3, gzip):
    # calling cartesian coordinates llh is so wrong
    args = {'station': station, 'isnr': snr,  'overwrite': overwrite, 'dec':dec, 'llh': llh, 'recv': recv, 'sp3': sp3, 'gzip': gzip, 'orb': orb, 'hour': hour}

    # first get it working without parallel processing
    if not par:
        print('No parallel processing')
        mjd_list = {}; mjd_list[0] = [MJD1, MJD2]
        s1 = time.time()
        process_jobs_multi(index=0,args=args,datelist=mjd_list,error_queue=error_queue)
        s2 = time.time()
        print('That took ', round(s2-s1,2), ' seconds')
    else:
        if par > 10:
            print('Only allow ten parallel processes for now')
            par = 10
        numproc = par

        # get a list of times in MJD associated with the multiple spawned processes
        # this does not work for skipping dates though ...
        datelist, numproc = guts2.make_parallel_proc_lists_mjd(year, doy, year_end, doy_end, numproc)
        print(datelist)

        # make a list of process IDs
        index = list(range(numproc))
        s1 = time.time()

        pool = multiprocessing.Pool(processes=numproc)

        partial_process = partial(process_jobs_multi, args=args,datelist=datelist,error_queue = error_queue)

        pool.map(partial_process,index)

        pool.close()
        pool.join()
        s2 = time.time()
        print('That took ', round(s2-s1,2), ' seconds')


def process_jobs_multi(index,args,datelist,error_queue):
    """
    runs the nmea2snr queue

    Parameters
    ----------
    index : int
        which job to run
    args : dict
        dictionary of parameters for run_nmea2snr
    datelist: dict
        start and stop dates in MJD
    error_queue:?
        not sure how to describe this

    """

    # should try be on each submission?  or each list of submissions?
    try:
        d1 = datelist[index][0]; d2 = datelist[index][1]
        mjd_list = list(range(d1, d2+1))
        for mjd in mjd_list:
            y, d = g.modjul_to_ydoy(mjd)
            args['year'] = y
            args['doy'] = d
            nmea.run_nmea2snr(**args)

    except Exception as e:
        print('Error of some kind processing year/doy ', y, d)
        error_queue.put(e)

    return


def main():
    args = parse_arguments()
    nmea2snr(**args)


if __name__ == "__main__":
    main()




import warnings
import argparse
import os
import subprocess
import sys
import time
import wget
import multiprocessing
from functools import partial

#import gnssrefl.gnssir as guts
import gnssrefl.gnssir_v2 as guts2
import gnssrefl.gps as g
import gnssrefl.refraction as refr

from gnssrefl.utils import str2bool


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("station", help="station name (4 ch)", type=str)
    parser.add_argument("year", help="year", type=int)
    parser.add_argument("doy", help="day of year", type=int)

    #print('gnssrefl version ', str(g.version('gnssrefl')))

    # optional inputs
    parser.add_argument("-snr", default=None, help="snr file ending, default is 66", type=int)
    parser.add_argument("-plt", default=None, help="plt to screen (True or False)", type=str)
    parser.add_argument("-fr", default=None, nargs="*",type=int, help="Frequency override for json, e.g. 1 101 for GPS and Glonass L1")
    parser.add_argument("-ampl", default=None, type=float, help="Min spectral amplitude ")
    parser.add_argument("-sat", default=None, type=int, help="Only look at this satellite")
    parser.add_argument("-doy_end", default=None, type=int, help="doy end")
    parser.add_argument("-year_end", default=None, type=int, help="year end")
    parser.add_argument("-azim1", default=None, type=int, help="lower limit azimuth (deg)")
    parser.add_argument("-azim2", default=None, type=int, help="upper limit azimuth (deg)")
    parser.add_argument("-nooverwrite", default=None, type=str, help="default is False, i.e. you will overwrite")
    parser.add_argument("-extension", type=str, help="extension for result file, useful for testing strategies")
    parser.add_argument("-compress", default=None, type=str, help="Boolean, xz compress SNR files after use")
    parser.add_argument("-gzip", default=None, type=str, help="Boolean, gzip SNR files after use. Default is True")
    parser.add_argument("-screenstats", default=None, type=str, help="Boolean, some stats printed to screen(default is False)")
    parser.add_argument("-delTmax", default=None, type=int, help="Allowed satellite arc length (minutes)")
    parser.add_argument("-e1", default=None, type=float, help="min elev angle (deg)")
    parser.add_argument("-e2", default=None, type=float, help="max elev angle (deg)")
    parser.add_argument("-mmdd", default=None, type=str, help="Boolean, add columns for month,day,hour,minute")
    parser.add_argument("-dec", default=1, type=int, help="decimate SNR file to this sampling rate before computing periodograms")
    parser.add_argument("-newarcs", default=None, type=str, help="This no longer has any meaning")
    parser.add_argument("-par", default=None, type=int, help="Number of processes to spawn (up to 10)")

    g.print_version_to_screen()
    print (sys.version)

    args = parser.parse_args().__dict__

    # convert all expected boolean inputs from strings to booleans
    boolean_args = ['plt', 'screenstats', 'nooverwrite', 'compress', 'screenstats', 'mmdd','gzip','newarcs']
    args = str2bool(args, boolean_args)

    # only return a dictionary of arguments that were added from the user - all other defaults will be set in code below
    return {key: value for key, value in args.items() if value is not None}


def gnssir(station: str, year: int, doy: int, snr: int = 66, plt: bool = False, fr: list= [], 
        ampl: float = None, sat: int = None, doy_end: int = None, year_end: int = None, azim1: int = 0, 
        azim2: int = 360, nooverwrite: bool = False, extension: str = '', compress: bool = False, 
        screenstats: bool = False, delTmax: int = None, e1: float = None, e2: float = None, 
           mmdd: bool = False, gzip: bool = True, dec : int = 1, newarcs : bool = True, par : int = None ):
    """
    gnssir is the main driver for estimating reflector heights. The user is required to 
    have set up an analysis strategy using gnssir_input. 

    beta version of parallel processing is now onine. If you set -par to an integer between 2 and 10,
    it should substantially speed up your processing. Big thank you to AaryanRampal for getting this up and running.
        
    Examples
    --------
    gnssir p041 2021 15 
        analyzes the data for station p041, year 2021 and day of year 15.
    gnssir p041 2021 1 -doy_end 100 -par 10
        analyze 100 days of data - but spawn 10 processes at a time. Big cpu time savings.
    gnssir p041 2021 15  -snr 99
        uses SNR files with a 99 suffix
    gnssir p041 2021 15  -plt T
        plots of SNR data and periodograms come to the screen. Each frequency gets its own plot.
    gnssir p041 2021 15  -screenstats T
        sends debugging information to the screen
    gnssir p041 2021 15  -nooverwrite T 
        only runs gnssir if there isn't a previous solution
    gnssir p041 2021 15  -extension strategy1
        runs gnssir using json file called p041.strategy1.json
    gnssir p041 2021 15  -doy_end 20 
        Analyzes data from day of year 15 to day of year 20
    gnssir p041 2021 15 -dec 5
        before computing periodograms, decimates the SNR file contents to 5 seconds
    gnssir p041 2021 15 -gzip T
        gzips the SNR file after you run the code. Big space saver (now the default)
    gnssir p041 2021 15 -fr 1 101 
        ignore frequency list in your json and use frequencies 1 and 101 

    Parameters
    ----------
    station : str
        lowercase 4 character ID of the station
    year : int
        full Year
    doy : integer
        Day of year

    snr : int, optional
        SNR format. This tells the code what elevation angles to save data for. Input is the snr file ending.
        Value options:

            66 (default) : saves all data with elevation angles less than 30 degress

            99 : saves all data with elevation angles between 5 and 30 degrees

            88 : saves all data 

            50 : saves all data with elevation angles less than 10 degrees

    plt : bool, optional
        Send plots to screen or not. Default is False.
    fr : int, optional
        As of version 3.5.9 you are allowed to enter more than one GNSS frequency.
        It overrides whatever was stored in your json. This is entered into
        a list so do not use commas, i.e. 1 101 102 
        allowed frequency names:

            1,2,20,5 : GPS L1, L2, L2C, L5

            101,102 : GLONASS L1, L2

            201, 205,206,207,208 : GALILEO E1, E5a,E6,E5b,E5

            302,306,307 : BEIDOU B1, B3, B2 (not sure we do 307)

    ampl : float, optional
        minimum spectral peak amplitude. default is None
    sat : int, optional
        satellite number to only look at that single satellite. default is None.
    doy_end : int, optional
        end day of year. This is to create a range from doy to doy_end of days.
        If year_end parameter is used - then day_end will end in the day of the year_end.
        Default is None. (meaning only a single day using the doy parameter)
    year_end : int, optional
        end year. This is to create a range from year to year_end to get the snr files for more than one year.
        doy_end will be for year_end. Default is None.
    azim1 : int, optional
        lower limit azimuth.
        If the azimuth angles are changed in the json (using 'azval2' key) and not here, then the json overrides these.
        If changed here, then it overrides what you requested in the json. default is 0.
    azim2 : int, optional
        upper limit azimuth.
        If the azimuth angles are changed in the json (using 'azval2' key) and not changed here, then the json overrides these.
        If changed here, then it overrides what you requested in the json. default is 360.
    nooverwrite : bool, optional
        Use to overwrite lomb scargle result files or not.
        Default is False, i.e., it will overwrite.
    extension : string, optional
        extension for result file, useful for testing strategies. default is empty string
    compress : boolean, optional
        xz compress SNR files after use. default is False.
    screenstats : bool, optional
        whether to print stats to the screen or not. default is True.
    delTmax : int, optional
        maximum satellite arc length in minutes. found in the json
    e1 : float, optional
        use to override the minimum elevation angle.
    e2 : float, optional
        use to override the maximum elevation angle.
    mmdd : boolean, optional
        adds columns in results for month, day, hour, and minute. default is False.
    gzip : boolean, optional
        gzip compress SNR files after use. default is True (as of 2023 Sep 17).
    dec : int, optional
        decimate SNR file to this sampling period before the 
        periodograms are computed. 1 sec is default (i.e. no decimating)
    newarcs : bool, optional
        this input no longer has any meaning 
    par : int, optional
        number of parallel processing jobs. 

    """
    vers = 'gnssrefl version ' + str(g.version('gnssrefl'))
    print('You are running ', vers)


#   make sure environment variables exist.  set to current directory if not
    g.check_environ_variables()
    xdir = str(os.environ['REFL_CODE'])

    exitS = g.check_inputs(station, year, doy, snr)

    if exitS:
        sys.exit()

    # check this now before spawning multi-processor jobs
    if year_end is None:
        g.result_directories(station,year,extension)
    else:
        for y in range(year, year_end+1):
            g.result_directories(station,y,extension)

    lsp = guts2.read_json_file(station, extension)
    if 'snr' in lsp:
        snr = lsp['snr']
        print('Found a snr choice in the json:', snr)

    # make a refraction file you will need later
    refr.readWrite_gpt2_1w(xdir, station, lsp['lat'], lsp['lon'])

    # now check the overrides to the json instructions

    # requiring people use the new code
    if 'azval2' not in lsp:
        print('An azval2 variable was not found in your gnssir json input file. Fix your json ')
        print('and/or use gnssir_input to make a new one. Exiting')
        sys.exit()

    # plt is False unless user changes
    lsp['plt_screen'] = plt

    if delTmax is not None:
        lsp['delTmax'] = delTmax

    if ((lsp['maxH'] - lsp['minH']) < 5):
        print('Requested reflector heights (', lsp['minH'], ',', lsp['maxH'], ') are too close together. Exiting.')
        print('They must be at least 5 meters apart - and preferably further than that.')
        sys.exit()

    # compress is False unless user changes
    lsp['wantCompression'] = compress

    # screenstats is True unless user changes
    lsp['screenstats'] = screenstats

    # added this for people that have 1 sec files that do not need this resolution
    # decimation is a bit slow but way faster than doing a gazillion periodograms with 
    # 1 sec data

    lsp['dec'] = dec

    # in case you want to analyze multiple days of data
    if doy_end is None:
        doy_end = doy

    # mmdd is False unless user changes
    add_mmddhhss = mmdd


    # in case you want to analyze only data within one year
    year_st = year  # renaming for confusing variable naming when dealing with year calculations below
    if year_end is None:
        year_end = year

    # default will be to overwrite
    #if nooverwrite is None:
    lsp['nooverwrite'] = nooverwrite
    #else:
    #    lsp['overwriteResults'] = False

    if e1 is not None:
        lsp['e1'] = e1
    if e2 is not None:
        lsp['e2'] = e2

    # in case you want to look at a restricted azimuth range from the command line
    setA = 0
    # if azim1 is not default (has been changed)
    if azim1 != 0:
        setA = 1

    # if azim2 is not default (has been changed)
    if azim2 != 360:
        setA = setA + 1

    # TODO figure out the goal here
    # this only sets the new azim values only if bother azim1 and azim2 changed?
    if setA == 2:
        lsp['azval2'] = [azim1,  azim2]

    # this is for when you want to run the code with just a single frequency, i.e. input at the console
    # rather than using the input restrictions
    #print(lsp['freqs'])
    #if fr is not None:
    if len(fr) > 0:
        lsp['freqs'] = fr
        # better make sure you have enough amplitudes
        ampl_from_json = lsp['reqAmp'][0]
        if ampl is None:
            lsp['reqAmp'] = [ampl_from_json for i in range(14)]

    if ampl is not None:
        # this is not elegant - but allows people to set ampl on the command line
        # but use the frequency list from their json ...  which i think has max of 12
        # but use 14 to be sure
        lsp['reqAmp'] = [ampl for i in range(14)]

    if sat is not None:
        lsp['onesat'] = [sat]

    lsp['mmdd'] = add_mmddhhss
    # added 2022apr15
    lsp['gzip'] = gzip

    # if refraction model is not assigned, set it to 1
    if 'refr_model' not in lsp.keys():
        lsp['refr_model'] = 1

    # good lord - why is this here? surely a function can be called
    picklefile = 'gpt_1wA.pickle'
    pname = xdir + '/input/' + picklefile

    #    print('refraction file exists')
    if not os.path.isfile(pname):
        local_copy = 'gnssrefl/' + picklefile
        if os.path.isfile(local_copy):
            print('found local copy of refraction file')
            subprocess.call(['cp', '-f', local_copy, xdir + '/input/'])
        else:
            print('download and move refraction file')
            url='https://github.com/kristinemlarson/gnssrefl/raw/master/gnssrefl/gpt_1wA.pickle'
            wget.download(url, picklefile)
            subprocess.call(['mv', '-f', picklefile, xdir + '/input/'])

    args = {'station': station.lower(), 'year': year, 'doy': doy, 'snr_type': snr, 'extension': extension, 'lsp': lsp}

    print(lsp['pele'], ' direct signal elevation angle limits')
    #print(lsp['e1'], lsp['e2'], ' min and max elevation angles')
    # added this because ellist is a new option and was not necessarily created in old json files
    if 'ellist' not in lsp:
        #print('did not find ellist')
        if float(lsp['e1']) < float(lsp['pele'][0]):
            print('emin is smaller than the minimum eangle (pele) used for direct signal removed.')
            print('This is Forbidden. Fix the records set in the json created by gnssir_analysis')
            sys.exit()
    else:
        if float(lsp['e1']) < float(lsp['pele'][0]):
            print('Your requested emin is lower than the minimum elevation angle used ')
            print('for direct signal removal. This is stored in the variable pele.')
            print('This is Forbidden. Fix the pele records set in the json created by gnssir_analysis')
            sys.exit()
        if float(lsp['e2']) > float(lsp['pele'][1]):
            print('Your requested emax is higher than the maximum elevation angle used ')
            print('for direct signal removal. This is stored in the variable pele.')
            print('This is Forbidden. Fix the pele records set in the json created by gnssir_analysis')
            sys.exit()

    # should make sure there are directories for the results ... 
    g.checkFiles(station.lower(), extension)

    #if screenstats:
    # for my sanity might as well print it out
    if True:
        print('Requested frequencies ', lsp['freqs'])
        #print('Requested amplitudes', lsp['reqAmp'])

    # queue which handles any exceptions any of the processes encounter
    manager = multiprocessing.Manager()
    error_queue = manager.Queue()
    additional_args = { "args": args, "error_queue": error_queue }

    t1 = time.time()
    if (year == year_end) & (doy == doy_end):
        if par:
            print('You are analyzing only one day of data - no reason to submit multiple processes')
            par = None

    if not par: 
        print('Parallel processing not requested')
        # this should be changed to call process_year_dictionary
        process_year(year,year_end,doy,doy_end, **additional_args)

    else:
        print('Parallel processing chosen')
        if par > 10:
            print('For now we will only allow ten simultaneous processes. Submit again. Exiting.')
            sys.exit()
        else:
            numproc = par
            print(year,doy,year_end,doy_end)
            # get a list of times in MJD associated with the multiple spawned processes
            d,numproc=guts2.make_parallel_proc_lists_mjd(year, doy, year_end, doy_end, numproc)

            # make a list of process IDs
            index_list = list(range(numproc))

            pool = multiprocessing.Pool(processes=numproc) 

            partial_process_yearD = partial(process_year_dictionary, args=args,datelist=d, error_queue = error_queue)

            pool.map(partial_process_yearD,index_list)

            pool.close()
            pool.join()


        if not error_queue.empty():
            print("One (or more) of the processes encountered errors. Will not proceed until errors are fixed.")
            i = 1
            while not error_queue.empty():
                e = error_queue.get()
                print(f"Error {i} type: {type(e)}. Error {i} message: {e}")
                i += 1
            sys.exit(1)

    t2 = time.time()
    print('Time to compute: ', round(t2-t1,2), ' seconds')


# process year and process year dictionary should be combined at some point
# see rinex2snr as example

def process_year(year, year_end, doy, doy_end, args, error_queue):
    """
    Code that does the processing for a specific year. Refactored to separate 
    function to allow for parallel processes

    Parameters
    ----------
    year : int
        the start year 
    year_end : int
        end year. This was the last year you plan to analyze
    doy : integer
        Day of year to start processing 
    doy_end : int
        end day of year on the last year you plan to analyze
    args : dict
        arguments passed into gnssir through commandline (or python)

    """
    # an infinitely better way
    MJD1 = int(g.ydoy2mjd(year,doy))
    MJD2 = int(g.ydoy2mjd(year_end,doy_end))
    for modjul in range(MJD1, MJD2+1):
        y, d = g.modjul_to_ydoy(modjul)
        args['year'] = y
        args['doy'] = d
        try:
            guts2.gnssir_guts_v2(**args)
        except:
            warnings.warn(f'error processing {y} {d}');                

    # where should i put the error queue statement?
    return

def process_year_dictionary(index,args,datelist,error_queue):
    """
    Code that does the processing for a specific year. Refactored to separate
    function to allow for parallel processes

    Parameters
    ----------
    index: int
        accommodation for parallel processing. It should be a value 
        from 0 to 9, telling the code which part of datelist to use 
    args : dict
        arguments passed into gnssir through commandline (or python)
        should have the new arguments for sublists
    datelist : dict
        list of dates you want to analyze in pairs of MJD
        could have up to 10 sets of dates, from 0 to 9, e.g. for two processes
        dd = { 0: [MJD1, MJD2], 1:[MJD1,MJD2] }
    error_queue : ?

    """
    #should the try be further in the loop?
    try:
        d1 = datelist[index][0]; d2 = datelist[index][1] 
        mjd_list = list(range(d1, d2+1))
 
        for MJD in mjd_list:
            year, doy = g.modjul_to_ydoy(MJD)
            args['year'] = year
            args['doy']=doy
            print(f'Processing MJD {MJD} Year {year} DOY {doy}');

            guts2.gnssir_guts_v2(**args)
    except Exception as e:
        error_queue.put(e)


def main():
    args = parse_arguments()
    gnssir(**args)


if __name__ == "__main__":
    main()


